<?php
namespace Controller;

use React\Stream\Util;
use Service\Appropriation;
use Service\Resource;
use Util\Messages;
use Util\Utilities;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\Log\Exception;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;

/**
 * Class Task
 *
 * Class responsible for receiving requests for screens that control tasks
 *
 * @package Controller
 */
class Task extends Simple {

	/** @var \Service\Task $taskService */
	/** @var \Service\Resource $resourceService */
	protected $resourceService;
	/** @var Appropriation $appropriationService */
    protected $appropriationService;

	protected $environment;

    const TASK_PREFIX = 'T';
    const PROJECT_PREFIX = 'P';

	public function __construct(\Service\Task $taskService, Resource $resourceService, Appropriation $appropriationService, \Util\Utilities $environment) {
		$this->taskService = $taskService;
		$this->resourceService = $resourceService;
        $this->appropriationService = $appropriationService;
		$this->environment = $environment;
	}

    public function findAllTaskByProject(Request\Filter $request,Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $projectId = empty($conditions[0]['value']) ? 0 : $conditions[0]['value'] ;
        $dataSet = $this->taskService->findAllTaskByProject($projectId);
        $response->addDataset(new DataSet('projects', $dataSet));
    }
    public function calculationBasedStart(Request\DataSet $request,Response $response){
        try {
            $row = $request->getDataSet()->getRows()[0];
            list($task, $begin, $end, $duration, $precursor, $precursorType, $parent, $exception, $exceptionDate,
                $projectId,$resourceIds,$latency) = $this->getValuesCalculation($row);
            list( $begin, $end, $exception, $exceptionDate) = $this->taskService->calculationBasedStart($task, $begin, $end, $duration,
                $precursor, $precursorType, $parent, $exception, $exceptionDate,$projectId,$resourceIds,$latency);
            $dataSet = array(array(
                'dateBegin' => $begin->format(Utilities::FORMAT_DATE_DEFAULT_ONLY_DATE),
                'dateEnd' => $end->format(Utilities::FORMAT_DATE_DEFAULT_ONLY_DATE),
                'exception' => $exception,
                'exceptionDate' => $exceptionDate,
            ));
            $response->addDataset(new DataSet('calculationDate', $dataSet));
        }catch (\Exception $e){
            $response->addMessage(new Response\Message('Erro inesperado '.$e->getMessage()));
            throw $e;
        }
    }
    public function calculationBasedEnd(Request\DataSet $request,Response $response){
        try {
            $row = $request->getDataSet()->getRows()[0];
            list($task, $begin, $end, $duration, $precursor, $precursorType, $parent, $exception, $exceptionDate,
                $projectId,$resourceIds,$latency) = $this->getValuesCalculation($row);
            list($begin,$end, $exception, $exceptionDate) = $this->taskService->calculationBasedEnd($task, $begin, $end, $duration,
                $precursor, $precursorType, $parent, $exception, $exceptionDate,$projectId,$resourceIds,$latency);
            $dataSet = array(array(
                'dateBegin' => $begin->format(Utilities::FORMAT_DATE_DEFAULT_ONLY_DATE),
                'dateEnd' => $end->format(Utilities::FORMAT_DATE_DEFAULT_ONLY_DATE),
                'exception' => $exception,
                'exceptionDate' => $exceptionDate,
            ));
            $response->addDataset(new DataSet('calculationDate', $dataSet));
        }catch (\Exception $e){
            $response->addMessage(new Response\Message('Erro inesperado '.$e->getMessage()));
            throw $e;
        }
    }

    /**
     * Finds all resources
     * @param Request\Filter $request
     * @param Response $response
     */
    public function findResource(Request\Filter $request, Response $response) {
		$param = $request->getFilterCriteria()->getConditions();
        $filterstring = Utilities::getFilterTableColumn($param, 'NAME');
        $project = $param[0]['value'];
        $taskClassificationControl = Utilities::getValueInFilterDataSourceByName($param, 'TASK_CLASSIFICATION');
		$dataSet = $this->resourceService->getResourceByProject($project, $filterstring, $taskClassificationControl);
		$response->addDataSet(new DataSet('taskResource', $dataSet));
	}

    public function findTaskResourceMeeting(Request\Filter $request, Response $response) {
		$param = $request->getFilterCriteria()->getConditions();
		$project = Utilities::getValueInFilterDataSourceByName($param,'PROJECT');
		$ids = Utilities::getValueInFilterDataSourceByName($param,'ID');
		$dataSet = $this->resourceService->getTaskResourceMeeting($project,$ids);
		$response->addDataSet(new DataSet('taskResourceMeeting', $dataSet));
	}

    public function findParentTaskByProjectAndTask(Request\Filter $request, Response $response) {
		$param          = $request->getFilterCriteria()->getConditions();
		$filterString   = Utilities::getFilterTableColumn($param, 'NAME');
        $isParent       = DevOpsUtilities::getValueInFilterDataSourceByName($param,'IS_PARENT',true);
        $project        = DevOpsUtilities::getValueInFilterDataSourceByName($param,'PROJECT',true);
        $task           = DevOpsUtilities::getValueInFilterDataSourceByName($param,'TASK',true);
        $selectedTasks  = DevOpsUtilities::getValueInFilterDataSourceByName($param,'selectedTasksIds',true);

        if(!$project){
            $project = DevOpsUtilities::getValueInFilterDataSourceByName($param,'projectId',true);
        }else if(!$project){
            $project = '0';
        }

        if(!$task){
            $task = '0';
        }

        if(!$selectedTasks){
            $selectedTasks = array(0);
        }

		$dataSet        = $this->taskService->getParentTaskByProjectAndTask($project, $task, $selectedTasks,$isParent,$filterString);
		$response->addDataSet(new DataSet('taskParent', $dataSet));
	}
	public function findPrecursorTaskByProjectAndTask(Request\Filter $request, Response $response) {
		$param = $request->getFilterCriteria()->getConditions();
		$project = $param[0]['value'];
		$task = $param[1]['value'];

		$dataSet = $this->taskService->getPrecursorTaskByProjectAndTask($project, $task);
		$response->addDataSet(new DataSet('taskParent', $dataSet));
	}

	public function getDateTasksFromUser(Request\Filter $request, Response $response) {
		$taskDates = $this->taskService->getDateTasksByCurrentUser();
		$response->addDataSet(new DataSet('taskDates', $taskDates));
	}
	public function findAllStatusTask(Request\Filter $request, Response $response) {
        $conditions = $request->getFilterCriteria()->getConditions();
		$taskDates = $this->taskService->getAllStatusTask($conditions);
		$response->addDataSet(new DataSet('findAllStatusTask', $taskDates));
	}
    public function findAllTypeTask(Request\Filter $request, Response $response) {
		$taskDates = $this->taskService->getAllTypeTask();
		$response->addDataSet(new DataSet('taskDates', $taskDates));
	}

    public function saveNewTaskStatus(Request\DataSet $request, Response $response) {
        try{
            $rows = $request->getDataSet()->getRows();
            foreach ($rows as $row){
                $typeType = $row['TASK_TYPE'];
                $name = $row['NAME'];
                $this->taskService->saveNewTaskStatus($name,$typeType);
            }

        }catch (\Exception $e){
            $response->addMessage(new Response\Message('Erro inesperado '.$e->getMessage()));
        }
	}
    public function findTaskBranch(Request\Filter $request, Response $response){

        try {
                $param = $request->getFilterCriteria()->getConditions();
                $taskId = $param[0]['value'];
                $taskBranch = $this->taskService->findTaskBranch($taskId);
                $response->addDataset(new DataSet('hasBranch', $taskBranch));

            } catch (\Exception $e) {
                $response->addMessage(new Response\Message('Erro inesperado '.$e->getMessage()));
            }
    }
    public function getResourceTasks(Request\Filter $request, Response $response) {

       try {
                $tasks = $this->taskService->getTasksByUser($this->environment->getCurrentUserId());
                $response->addDataSet(new DataSet('resourceTasks', $tasks));
            } catch (\Exception $e) {
                $response->addMessage(new Response\Message('Erro inesperado '.$e->getMessage()));

            }

    }

    public function getTasksByUserFromDate(Request\Filter $request, Response $response) {
		$conditions = $request->getFilterCriteria()->getConditions();
        if($conditions[0]['value'] == '='){
            $fromDate = '';
        }else{
            $fromDate = $conditions[0]['value'];
        }
        $today = date("d/m/Y");
        if($fromDate == $today){
            $tasks = $this->taskService->getTasksByCurrentUserFromDate($fromDate);
            $delayedTasks = $this->taskService->getDelayedTasksByCurrentUser();
            $tasks = $this->joinTasks($tasks, $delayedTasks);
        }else{
            $tasks = $this->taskService->getTasksByCurrentUserFromDate($fromDate);
        }

        foreach ($tasks as $key => $task) {
            $tasks[$key]['APPROPRIATION'] = $this->appropriationService->findAppropriationByTask($task['ID']);
            $tasks[$key]['PATHS'] = $this->appropriationService->findPaths($task['ID'], $task['RESOURCE_T']);
        }

        $response->addDataSet(new DataSet('tasks', $tasks));

    }


    /************************************************************
     * Zeedhi Filter has a problem: we have 'columnName', 'operator'
     * and 'value'. However, if value is null, then value automatically
     * becomes the operator value. We don't want to get a date with
     * '=' or '<>' value.
     * @param array $dateFilter
     ************************************************************/
    protected function removeOperatorSignalFromNullValue($dateFilter){
        if($dateFilter == '=' || $dateFilter == '<>'){
            return '';
        }
        return $dateFilter;
    }


    /************************************************************
     * Gets all tasks by current date selected on stripe, located
     * in My Tasks (Minhas Tarefas) screen.
     * @param Request\Filter    $request
     * @param Response          $response
     ************************************************************/
    public function getTasksByUserFromDateOnDemand(Request\Filter $request, Response $response)
    {
        $conditions = $request->getFilterCriteria()->getConditions();
        $dateFilter = Utilities::getValueInFilterDataSourceByName($conditions, 'ESTIMATED_BEGIN_DATE');
        $fromDate = $this->removeOperatorSignalFromNullValue($dateFilter);
        $projectFilter = Utilities::getValueInFilterDataSourceByName($conditions, 'PROJECT_ID');

        $dataSet = $this->taskService->getTasksByUserFromDateOnDemand($fromDate, $projectFilter);

        DevopsUtilities::setDataSetDefault($request, $response, $dataSet);
    }

    public function recalculateTaskProgress(Request\DataSet $request, Response $response){
        $task = $request->getDataSet()->getRows();
        $id = $task[0]['ID'];
        $progress = $task[0]['PERCENT_COMPLETED'];
        try{
            $task = $this->taskService->recalculateTaskProgress($id, $progress);
            $response->addDataSet(new DataSet('recalculateTaskProgress', array($task)));
        } catch (\Exception $e){
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_WARNING));
        }
    }

	private function joinTasks($tasks, $delayedTasks) {
        $tasks = array_merge($tasks, $delayedTasks);
		return $tasks;

	}

	public function findTaskByProjectGantt(Request\Filter $request, Response $response) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $projectId = Utilities::getValueInFilterDataSourceByName($conditions, 'project',true);
        $showCompletedTasks = Utilities::getValueInFilterDataSourceByName($conditions, 'showCompletedTasks',true);
        $tasks = $this->taskService->getTaskByProjectGantt($projectId,$showCompletedTasks,$conditions);
        DevopsUtilities::setDataSetDefault($request, $response, $tasks);
	}

	public function findTaskByDate(Request\Filter $request, Response $response) {
		$filter = $request->getFilterCriteria()->getConditions();
        $date = $filter[0]['value'];

        $tasks = $this->taskService->getTaskByDate($date);
        $response->addDataSet(new DataSet("findTaskByDate", $tasks));
	}

	public function findPercentAppropriated(Request\Filter $request, Response $response) {
		$filter = $request->getFilterCriteria()->getConditions();
        $date = $filter[0]['value'];
        $percentAppropriated = $this->taskService->getPercentAppropriated($date);
        $percentAppropriated = array($percentAppropriated);
        $response->addDataSet(new DataSet("findPercentAppropriated", $percentAppropriated));
	}

    public function findErrorTaskFromErrorMerge(Request\Filter $request, Response $response) {
        $filter = $request->getFilterCriteria()->getConditions();
        $isFromTest = $filter[0]['value'];
        $test = $this->taskService->findErrorTaskFromErrorMerge($isFromTest);
        $response->addDataSet(new DataSet("findIfIsFromTest", $test));

    }

	public function findMergeRequestByTask(Request\Filter $request, Response $response) {
        $conditions   = $request->getFilterCriteria()->getConditions();
        $taskIds      = $this->environment->getValueInFilterDataSourceByName($conditions, 'TASK');
        $mergeRequest = $this->taskService->findMergeRequestByTask($taskIds);
        $response->addDataSet(new DataSet("findMergeRequestByTask", $mergeRequest));
	}

    public function updateTaskClassification(Request\Dataset $request, Response $response) {
        $rowWithTaskToUpdate = $request->getDataSet()->getRows();
        $taskId = $rowWithTaskToUpdate[0]['ID'];
        $taskClassificationId = $rowWithTaskToUpdate[0]['CLASSIFICATION_ID'];
        try {
            $this->taskService->updateTaskClassification($taskId, $taskClassificationId);
            $response->addDataSet(new DataSet('updateTaskStatus', $rowWithTaskToUpdate));
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function updateTaskStatus(Request\DataSet $request, Response $response) {
        $rowsWithTaskToUpdated = $request->getDataSet()->getRows();
        $taskId = $rowsWithTaskToUpdated[0]['TASK_ID'];
        $taskStatusControl = $rowsWithTaskToUpdated[0]['TASK_STATUS_CONTROL'];
        $allocation = $rowsWithTaskToUpdated[0]['ALLOCATION'];
        $initTask = empty($rowsWithTaskToUpdated[0]['INIT_DATE']) ? null : $rowsWithTaskToUpdated[0]['INIT_DATE'];
        $taskResponse = empty($rowsWithTaskToUpdated[0]['TASK_RESPONSE']) ? null : $rowsWithTaskToUpdated[0]['TASK_RESPONSE'];
        $resource = empty($rowsWithTaskToUpdated[0]['RESOURCE']) ? null : $rowsWithTaskToUpdated[0]['RESOURCE'];

        try {
            $this->taskService->updateTaskStatusByControl($taskId, $taskStatusControl, $initTask, $allocation, $taskResponse, $resource);
            $response->addDataSet(new DataSet('updateTaskStatus', $rowsWithTaskToUpdated));
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /*  Loads all task status registered in
        database.
    */
    public function loadAllTaskStatus(Request\Filter $request, Response $response) {
        $taskStatus = $this->taskService->getAllTaskStatus();
        $response->addDataSet(new DataSet('taskStatus', $taskStatus));
    }

    public function getUserTasks(Request\Filter $request, Response $response){
        $tasks = $this->taskService->getUserTasks();
        $response->addDataSet(new DataSet('tasks', $tasks));
    }

    public function getUserTasksName(Request\Filter $request, Response $response){
        $param = $request->getFilterCriteria()->getConditions();
        $project = $param[0]['value'];
        $tasks = $this->taskService->getUserTasksName($project);
        $response->addDataSet(new DataSet('tasksName', $tasks));
    }

    public function getTaskData(Request\Filter $request, Response $response) {
        $taskId = $request->getFilterCriteria()->getConditions();
        $taskId = $taskId[0]["value"];

        $taskData = $this->taskService->getTaskData($taskId);
        $response->addDataSet(new DataSet('findTaskData', $taskData));
    }

    public function getTaskAppropriations(Request\Filter $request, Response $response) {
        $taskId = $request->getFilterCriteria()->getConditions();
        $taskId = $taskId[0]["value"];

        $taskData = $this->appropriationService->findAppropriationByTask($taskId);
        $response->addDataSet(new DataSet('findTaskAppropriations', $taskData));
    }

    public function findTaskAppropriationsTimeControl(Request\Filter $request, Response $response) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $taskId = $this->environment->getValueInFilterDataSourceByName($conditions, 'taskId', true);
        $filters = $this->environment->createQueryFiltered($conditions);
        $taskData = $this->appropriationService->getTaskAppropriationsTimeControl($taskId, $filters);
        DevopsUtilities::setDataSetDefault($request, $response, $taskData);
    }

    public function findIssueStatus(Request\Filter $request, Response $response) {
        try {
            $filter = $request->getFilterCriteria()->getConditions();
            if(isset($filter[count($filter) - 1]['operator']) &&
               $filter[count($filter) - 1]['operator'] == 'LIKE_ALL'){
                $filterName = $filter[count($filter) - 1]['columnName'];
                $filterOperator = "LIKE";
                $filterValue = $filter[count($filter) - 1]['value'];
            } else {
                $filterName = false;
                $filterOperator = false;
                $filterValue = false;
            }

            $issueStatus = $this->taskService->findIssueStatus($filterName, $filterOperator, $filterValue);
            $response->addDataSet(new DataSet('findIssueStatus', $issueStatus));
        }catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_WARNING));
        }
    }

    public function findIssueClassification(Request\Filter $request, Response $response) {
        try {
            $filter = $request->getFilterCriteria()->getConditions();
            $filterString = DevOpsUtilities::createQueryFiltered($filter);
            $issueClassification = $this->taskService->findIssueClassification($filterString);
            $response->addDataSet(new DataSet('findIssueClassification', $issueClassification));
        }catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_WARNING));
        }
    }

    public function saveNewIssueStatus(Request\DataSet $request, Response $response) {
        $dataSet = $request->getDataSet()->getRows();
        $param = $dataSet[0];

        $label = $param["LABEL"];
        $description = empty($param["DESCRIPTION"]) ? "" : $param["DESCRIPTION"];
        $issueType = $param["ISSUE_TYPE"];

        if(!empty($label) && !empty($issueType)) { // Se o nome e issueType for valido
            if($this->taskService->saveNewIssueStatus($label, $description, $issueType))
                $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_SAVED));
            else
                $response->addMessage(new Response\Message("Error: Issue Status"));
        }
        else {
            $response->addMessage(new Response\Message("Nome inválido"));
        }
    }

    public function saveNewIssueType(Request\DataSet $request, Response $response) {
        $dataSet = $request->getDataSet()->getRows();
        $param = $dataSet[0];

        $label = $param["LABEL"];
        $description = empty($param["DESCRIPTION"]) ? "" : $param["DESCRIPTION"];
        $board = $param["BOARD"];

        if(!empty($label)) { // Se a label for valida
            if($this->taskService->saveNewIssueType($label, $description, $board))
                $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_SAVED));
            else
                $response->addMessage(new Response\Message("Error: Issue Type"));
        }
        else {
            $response->addMessage(new Response\Message("Nome inválido"));
        }
    }

    /**
     * Saves a new issue from Issue Tracker when
     * user is on 'My Issues' tab.
     * @param Request\DataSet $request
     * @param Response $response
     */
    public function saveNewIssue(Request\DataSet $request, Response $response) {
        $dataSet = $request->getDataSet()->getRows();
        $param = $dataSet[0];
        $name           = $param["LABEL"];
        $classification = $param["ISSUE_CLASSIFICATION"];
        $issueStatus    = empty($param["ISSUE_STATUS"]) ? null : $param["ISSUE_STATUS"];
        $reporter       = empty($param["REPORTER"]) ? null : $param["REPORTER"];
        $dueDate        = empty($param["DUE_DATE"]) ? "" : $param["DUE_DATE"];
        $endDate        = empty($param["END_DATE"]) ? null : $param["END_DATE"];
        $priority       = empty($param["PRIORITY"]) ? null : $param["PRIORITY"];
        $percentCompleted = empty($param["PERCENT_COMPLETED"]) ? 0 : $param["PERCENT_COMPLETED"];
        $description    = empty($param["DESCRIPTION"]) ? "" : $param["DESCRIPTION"];
        $board          = $param["BOARD"];
        $followers      = empty($param["FOLLOWERS"]) ? array() : $param['FOLLOWERS'];
        $sponsor        = empty($param['SPONSOR_ID']) ? null : $param['SPONSOR_ID'];
        $productVersionId = empty($param['PRODUCT_VERSION_ID']) ? null : $param['PRODUCT_VERSION_ID'];
        $version        = empty($param['VERSION']) ? null : $param['VERSION'];
        $productVersionDevId = empty($param['PRODUCT_VERSION_DEV_ID']) ? null : $param['PRODUCT_VERSION_DEV_ID'];
        $versionDev        = empty($param['VERSION_DEV']) ? null : $param['VERSION_DEV'];
        $projectId        = empty($param['PRODUCT']) ? null : $param['PRODUCT'];
        $isBoardIssue     = empty($param['IS_BOARD_ISSUE']) ? null : $param['IS_BOARD_ISSUE'];
        if(isset($param["LABELS"])){
            $labels = is_array($param["LABELS"]) ? $param["LABELS"] : null;
        }else{
            $labels = null;
        }

        try{
            $this->taskService->saveNewIssue($name, $classification, $issueStatus, $reporter, $dueDate, $priority, $percentCompleted, $description, $board, $followers, $labels, $sponsor, $endDate, $productVersionId, $version, $projectId, $versionDev, $productVersionDevId, $isBoardIssue);
            $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_SAVED));
        }
        catch(\Exception $e){
            $response->addMessage(new Response\Message(Messages::ERR_NOT_SUCCESSFUL_SAVED_EXPLANATION . $e->getMessage()));
        }
    }

    public function saveEditedIssue(Request\DataSet $request, Response $response) {
        $dataSet = $request->getDataSet()->getRows();
        $param = $dataSet[0];
        $newIssueType = $param["newIssueType"];
        $issueId = $param["issueId"];
        $description = isset($param['DESCRIPTION']) ? $param['DESCRIPTION'] : null;
        $sendMail = isset($param['sendMail']) ? $param['sendMail'] : false;
        $this->taskService->saveEditedIssue($newIssueType, $issueId, $sendMail, $description);
    }

    public function findIssueType(Request\Filter $request, Response $response) {
        $dataSet = $request->getFilterCriteria()->getConditions();
        $boardId = $dataSet[0]["value"];

        $issueType = $this->taskService->findIssueType($boardId);
        $response->addDataSet(new DataSet('findIssueType', $issueType));
    }

    public function findKanbanItems(Request\Filter $request, Response $response) {
        $filters = $request->getFilterCriteria()->getConditions();
        $boardId = Utilities::getValueInFilterDataSourceByName($filters, 'boardId', true);
        $structureIds = Utilities::getValueInFilterDataSourceByName($filters, 'STRUCTURE', true);
        $projectStatusIds = Utilities::getValueInFilterDataSourceByName($filters, 'PROJECT_STATUS_ID', true);
        Utilities::getValueInFilterDataSourceByName($filters, 'PRODUCT', true);
        $kanbanItems = $this->taskService->findKanbanItems($boardId, $filters, $structureIds, $projectStatusIds);
        $response->addDataSet(new DataSet('issues', $kanbanItems));
    }

    public function findAllIssues(Request\Filter $request, Response $response) {
        try{
            $filters = $request->getFilterCriteria()->getConditions();
            $kanbanItems = $this->taskService->findAllIssues($filters);
            $response->addDataSet(new DataSet('issues', $kanbanItems));
        } catch (\Exception $ex) {
            $response->addMessage(new Response\Message($ex->getMessage(), Response\Message::TYPE_WARNING));
        }
    }

    public function findAllIssuesGrid(Request\Filter $request, Response $response)
    {
        $filter  = $request->getFilterCriteria()->getConditions();
        $dataSet = $this->taskService->findAllIssuesGrid($filter);
        DevOpsUtilities::setDataSetDefault($request, $response, $dataSet);
    }

    public function findAllIssuesFromFilter(Request\Filter $request, Response $response) {
        $dataSet = $request->getFilterCriteria()->getConditions();
        $kanbanItems = $this->taskService->findAllIssuesFromFilter($dataSet);
        $response->addDataSet(new DataSet('issues', $kanbanItems));
    }

    /**
     * Gets all issues created by that user.
     * @param Request\Filter $request
     * @param Response $response
     */
    public function findMyKanbanItems(Request\Filter $request, Response $response) {
        $filters = $request->getFilterCriteria()->getConditions();
        $kanbanItems = $this->taskService->findMyKanbanItems($filters);
        $response->addDataSet(new DataSet('issues', $kanbanItems));
    }

    public function findMyProductsIssuesUnread(Request\Filter $request, Response $response)
    {
        $page = $request->getFilterCriteria()->getPage();
        $pageSize = $request->getFilterCriteria()->getPageSize();
        $filters = $request->getFilterCriteria()->getConditions();
        $dataSet = $this->taskService->findMyProductsIssuesUnread($filters, $page, $pageSize);
        DevOpsUtilities::setDataSetDefault($request, $response, $dataSet);
    }

    public function findMyIssuesUnread(Request\Filter $request, Response $response) {
        $filters = $request->getFilterCriteria()->getConditions();
        $page = $request->getFilterCriteria()->getPage();
        $pageSize = $request->getFilterCriteria()->getPageSize();
        $dataSet = $this->taskService->findMyIssuesUnread($filters, $page, $pageSize);
        DevOpsUtilities::setDataSetDefault($request, $response, $dataSet);
    }

    public function totalCommentsUnread(Request\Filter $request, Response $response) {
        $filters = $request->getFilterCriteria()->getConditions();
        $total = $this->taskService->totalCommentsUnread($filters);
        $response->addDataSet(new DataSet('total', $total));
    }

    public function findProductsByManager(Request\Filter $request, Response $response) {
        $filters = $request->getFilterCriteria()->getConditions();
        $products = $this->taskService->findProductsByManager($filters);
        $response->addDataSet(new DataSet('products', $products));
    }

    public function findFollowedKanbanItems(Request\Filter $request, Response $response) {
        $filters = $request->getFilterCriteria()->getConditions();
        $kanbanItems = $this->taskService->findFollowedKanbanItems($filters);
        $response->addDataSet(new DataSet('issues', $kanbanItems));
    }

    public function findAllTasksByProject(Request\Filter $request, Response $response) {
        ini_set('max_execution_time','600');
        set_time_limit(600);
        $filters = $request->getFilterCriteria()->getConditions();
        if(empty($filters)){
            return 0;
        }
        $tasks = $this->taskService->findAllTasksByProject($filters);
        $response->addDataSet(new DataSet('findAllTasksByProject', $tasks));
    }

    public function findDefaultTaskTypes(Request\Filter $request, Response $response) {
        $taskTypes = $this->taskService->getDefaultTaskTypes();
        $response->addDataSet(new DataSet('taskTypes', $taskTypes));
    }

    public function findAllTaskTypes(Request\Filter $request, Response $response) {
        $filters = $request->getFilterCriteria()->getConditions();
        $id = Utilities::getValueInFilterDataSourceByName($filters, 'ID');
        if(is_null($id)){
            Utilities::getValueInFilterDataSourceByName($filters, 'ID', true);
        }
        $taskTypes = $this->taskService->getTaskTypes($filters);
        $response->addDataset(new DataSet('taskTypes', $taskTypes));
    }

    public function closeIssue(Request\DataSet $request, Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            $this->taskService->closeIssue($rows[0]['ISSUES']);
            $response->addNotification(new Response\Notification('Issue finalizada com sucesso'));
        }catch (\Exception $e){
            $response->addMessage(new Response\Message('Erro : '.$e->getMessage(),Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param Request\DataSet $request
     * @param Response $response
     */
    public function reopenIssue(Request\DataSet $request, Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            $this->taskService->reopenIssue($rows[0]['ISSUES']);
            $response->addNotification(new Response\Notification('Issue recusada com sucesso'));
        }catch (\Exception $e){
            $response->addMessage(new Response\Message('Erro : '.$e->getMessage(),Response\Message::TYPE_ERROR));
        }
    }


    /************************************************************
     * Gets all infos about a selected task, when user clicks
     * on a task on Projects (Projetos) screen.
     * @param Request\Filter    $request
     * @param Response          $response
     ************************************************************/
    public function dataEditTask( Request\Filter $request,Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();
            $idTask = $params[0]['value'];
            $dataSet = $this->taskService->getDataSetForEditTask($idTask);
            $response->addDataset(new DataSet('taskData', $dataSet));
        }catch (\Exception $e){
            $response->addMessage(new Response\Message('Não foi possível ler as informações da tarefa, pois '.$e->getMessage(),Response\Message::TYPE_WARNING));
        }
    }

    public function infoTaskInTaskTracking( Request\Filter $request,Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();
            $idTask = $params[0]['value'];
            $resource = DevOpsUtilities::getValueInFilterDataSourceByName($params,'RESOURCE_T');
            $dataSet = $this->taskService->getInfoTaskInTaskTracking($idTask,$resource);
            $response->addDataset(new DataSet('taskData', $dataSet));
        }catch (\Exception $e){
            $response->addMessage(new Response\Message('Não foi possível ler as informações da tarefa, pois '.$e->getMessage(),Response\Message::TYPE_WARNING));
        }
    }

    public function projectTasks(Request\Filter $request, Response $response){
        $params = $request->getFilterCriteria()->getConditions();
        $projectId = Utilities::getValueInFilterDataSourceByName($params, 'project', true);
        if($projectId) {
            $filter = Utilities::createQueryFiltered($params);
            $task = $this->taskService->getDataSetForEditProjectTask($projectId, $filter);
            $task = Utilities::fixParentsForTreeGrid($task,'ID','ID_PARENT');
            $response->addDataSet(new DataSet('projectTasks', $task));
        }
    }

    public function projectAllTasks(Request\Filter $request, Response $response){
        ini_set('max_execution_time','60');
        set_time_limit(60);
        $param = $request->getFilterCriteria()->getConditions();
        $projectId = $param[0]['value'];
        $projectId = implode(",",$projectId);
        $taskId = $param[1]['value'];
        $classificationId = empty($param[2]) ? null : $param[2]['value'];
        if(!empty($param[3])){
            $allFieldsField = array($param[3]);
        }else{
            $allFieldsField = null;
        }
        $itemsPerPage = $request->getFilterCriteria()->getPageSize();
        $page = $request->getFilterCriteria()->getPage();
        $resources = $this->taskService->getAllTasksUnderProject($projectId, $taskId, $classificationId,$allFieldsField);
        $resources = array_slice($resources, ($page-1)*$itemsPerPage, $itemsPerPage);
        $response->addDataSet(new DataSet('tasks', $resources));
    }

    public function deleteProjectTask(Request\DataSet $request,Response $response)
    {
        try {
            $data = $request->getDataSet()->getRows();
            $param = $data[0];
            foreach($param['TASKS'] as $task){
                if(!empty($task['ID'])){
                    $this->taskService->deleteTaskById($task['ID']);
                }
            }
            $response->addNotification(new Response\Notification('Tarefa apagado com sucesso.'));
        } catch (\Exception $ex) {
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
            $response->addMessage(new Response\Message($ex->getMessage(), Response\Message::TYPE_WARNING));
        }
    }

    /***********************************************************************
     * Saves a task.
     * Be careful: this method is also used by DevOps.
     ***********************************************************************/
    public function saveProjectTask(Request\DataSet $request, Response $response){
        $data = $request->getDataSet()->getRows();
        try{
            $param = $data[0];
            $validationSave = $this->validateSaveProjectTask($param['PROJECT'], $param['PARENT']);
            if($validationSave) {
                $taskEntity = $this->persistProjectTask($param);
                $taskPaths  = $this->appropriationService->findPaths($taskEntity->getId(), $this->environment->getCurrentUserId());
                $response->addDataset(new DataSet("ID", array($taskEntity->getId(), $taskPaths)));
            } else {
                throw new \Exception(Messages::ERR_NOT_SAVE_TASK_INCOERENT_VALUES,1);
            }
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }

    }
    public function saveProjectTaskIssue(Request\DataSet $request, Response $response){
        $data = $request->getDataSet()->getRows();
        try{
            $param = $data[0];
            $param['PARENT'] = empty($param['PARENT']) ? null : $param['PARENT'];
            $validationSave = $this->validateSaveProjectTask($param['PROJECT'], $param['PARENT']);
            if($validationSave) {
                $taskEntity = $this->persistProjectTask($param);
                $this->taskService->createTaskRelationship($taskEntity->getId(), $param['ISSUE'],\Service\TaskPrecursor::RELATIONSHIP_ISSUE_TYPE);
                $this->taskService->applyOrderToIssueById($param['ISSUE'][0]);

                $response->addDataset(new DataSet("ID", array($taskEntity->getId())));
                $response->addNotification(new Response\Notification(count($param['ISSUE']) . ' issue(s) relacionada(s) a ' . 1 . ' tarefa(s)'));
            } else {
                $response->addNotification(new Response\Notification(Messages::ERR_NOT_SAVE_TASK_INCOERENT_VALUES, Response\Message::TYPE_ERROR));
            }
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(),Response\Message::TYPE_ERROR));
        }

    }

    public function persistProjectTask($task){
        try{
            $project            = $task['PROJECT'];
            $duration           = empty($task['TOTAL_TIME']) ? '1D' : $task['TOTAL_TIME'];
            $latency            = empty($task['LATENCY']) ? null : $task['LATENCY'];
            $name               = $task['NAME'];
            $estimatedBeginDate = Utilities::createDateProjectTask($task['START'], $task['END']);
            $estimatedEndDate   = Utilities::createDateProjectTask($task['END'], $task['START']);
            $status             = empty($task['STATUS']) ? 'A' : $task['STATUS'];
            $description        = empty($task['DESCRIPTION']) ? null : $task['DESCRIPTION'];
            $percentCompleted   = empty($task['PERCENT_COMPLETED']) ? 0 : (($task['PERCENT_COMPLETED'] < 100) ? $task['PERCENT_COMPLETED'] : 100);
            $resource           = empty($task['RESOURCE']) ? array() : $task['RESOURCE'];
            $precursor          = empty($task['PRECURSOR_NAME']) ? null : $task['PRECURSOR'];
            $dateExceptionType  = empty($task['DATE_EXCEPTION_TYPE']) ? 'MB' : $task['DATE_EXCEPTION_TYPE'];
            $dateException      = empty($task['DATE_EXCEPTION']) ? null : \DateTime::createFromFormat('d/m/Y', $task['DATE_EXCEPTION']);
            $precursorType      = empty($task['TYPE']) ? null : $task['TYPE'];
            $taskStatus         = empty($task['TASK_STATUS']) ? null : $task['TASK_STATUS'];
            $taskId             = empty($task['ID']) ? null : $task['ID'];
            $parent             = empty($task['PARENT']) ? null : $task['PARENT'];
            $isParent           = empty($task['IS_PARENT']) ? 'N' : $task['IS_PARENT'];
            $urlToTest          = empty($task['URL_TO_TEST']) ? null : $task['URL_TO_TEST'];
            $classification     = empty($task['TASK_CLASSIFICATION']) ? null : $task['TASK_CLASSIFICATION'];
            $response           = empty($task['TASK_RESPONSE']) ? null : $task['TASK_RESPONSE'];
            $generatorTask      = empty($task['TASK_GENERATOR']) ? null : $task['TASK_GENERATOR'];
            $generatorTaskType  = empty($task['TASK_GENERATOR_RELATIONSHIP']) ? null : $task['TASK_GENERATOR_RELATIONSHIP'];
            $taskFeatures       = empty($task['TASK_FEATURES_ID']) ? null : $task['TASK_FEATURES_ID'];
            $timePlanned        = empty($task['TIME_PLANNED']) ? null : $task['TIME_PLANNED'];
            $taskLabel          = empty($task['TASK_LABEL']) ? array() : $task['TASK_LABEL'];
            $taskResponse       = empty($task['TASK_RESPONSE']) ? null : $task['TASK_RESPONSE'];
            $versionDevId       = empty($task['VERSION_DEV_ID']) ? null : $task['VERSION_DEV_ID'];

            $generatorTask      = (!empty($task['NOT_TASK_GENERATE']) && $task['NOT_TASK_GENERATE'] == 'N') ? null : $generatorTask;

            $task = $this->taskService->saveProjectTask($taskId, $project, $latency, $name, $estimatedBeginDate, $estimatedEndDate, $status,
                                                       $description, $percentCompleted, $duration, $resource, $precursor, $parent,
                                                       $precursorType, $taskStatus, $dateExceptionType, $dateException, $isParent, $urlToTest,
                                                       $classification, $response, $generatorTask, $generatorTaskType, $taskFeatures,
                                                       $timePlanned, $taskLabel, $taskResponse, $versionDevId);
        }catch (\Exception $e){
            throw $e;
        }
        return $task;
    }
    public function validateMergeTaskConclude(Request\Filter $request, Response $response){
        $param = $request->getFilterCriteria()->getConditions();
        $taskId = DevOpsUtilities::getValueInFilterDataSourceByName($param,'TASK');
        $projects = $this->taskService->validateMergeTaskConclude($taskId);
        DevOpsUtilities::setDataSetDefault($request,$response,$projects);
    }

    /**
     * @param $date
     * @return \DateTime
     */
    protected function createDateProject($date)
    {
        return strpos($date, '-') === false
            ? \DateTime::createFromFormat(Utilities::FORMAT_DATE_DEFAULT_ONLY_DATE,$date)
            : new \DateTime($date);
    }

    /**
     * valida se o tarefa superior tem o mesmo projeto que a tarefa
     * @param $project
     * @param $parent
     * @return bool
     */
    public function validateSaveProjectTask($project, $parent)
    {
        $validate = true;
        if (!empty($parent)) {
            $validate = $this->taskService->validateParentProjectTask($project, $parent);
        }
        return $validate;
    }

    public function deleteTaskAllocation(Request\DataSet $request, Response $response){
        $dataSet = $request->getDataSet()->getRows();
        try {
            $this->taskService->deleteAllocatedResources($dataSet[0]);
        } catch(\Exception $ex){
            $response->addMessage(new Response\Message(Messages::ERR_NOT_SUCCESSFUL_SAVED_EXPLANATION . $ex->getMessage()));
        }
        $response->addNotification(new Response\Notification("Recursos desalocados!"));
    }

    public function saveTasksAllocation(Request\DataSet $request, Response $response){
        $dataSet = $request->getDataSet()->getRows();
        $count = 0;
        foreach ($dataSet as $param) {
            $allocationType = $param['TYPE_ADD'];
            $resourceIds = $param['RESOURCES'];
            $taskIds = $param['TASK'];

            try {
                $count = $this->taskService->saveTasksAllocation($taskIds, $resourceIds, $allocationType);
            } catch(\Exception $e){
                $response->addMessage(new Response\Message(Messages::ERR_NOT_SUCCESSFUL_SAVED_EXPLANATION . $e->getMessage()));
            }
        }
        $response->addNotification(new Response\Notification("$count alocações salvas."));
    }

    public function redoBudget(Request\DataSet $request, Response $response)
    {
        $row = $request->getDataSet()->getRows()[0];
        $count = 0;

        $issueId = $row['ISSUE'];
        $description= $row['DESCRIPTION'];

        try {
            $this->taskService->redoBudget($issueId,$description);
        } catch (\Exception $e) {
            $response->setCriticalError(new Response\Error($e->getMessage(),$e->getCode()));
        }
    }

    /*******************************************************************
     * Gets informations about a specified issue.
     * @param Request\Filter $request
     * @param Response $response
     *******************************************************************/
    public function getIssueData(Request\Filter $request, Response $response) {
        $dataSet = $request->getFilterCriteria()->getConditions();
        $issueId = $dataSet[0]["value"];
        $issue = $this->taskService->findIssue($issueId);
        $response->addDataSet(new DataSet('issue', $issue));
    }
    /*******************************************************************
     * Gets informations about a specified issue.
     * @param Request\Filter $request
     * @param Response $response
     *******************************************************************/
    public function getAccompanyingIssueData(Request\Filter $request, Response $response) {
        $dataSet = $request->getFilterCriteria()->getConditions();
        $issueId = $dataSet[0]["value"];
        $issue = $this->taskService->findIssue($issueId,true);
        $response->addDataSet(new DataSet('issue', $issue));
    }

    public function saveNewTaskClassification(Request\DataSet $request, Response $response){
        $classification = $request->getDataSet()->getRows();
        $classification = $classification[0];
        try{
            $this->taskService->saveNewTaskClassification($classification['LABEL']);
            $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_SAVED));
        }
        catch(\Exception $e){
            $response->addMessage(new Response\Message(Messages::ERR_NOT_SUCCESSFUL_SAVED_EXPLANATION . $e->getMessage()));
        }
    }

    public function findTaskRelationship(Request\Filter $request, Response $response) {
        $dataSet = $request->getFilterCriteria()->getConditions();
        $taskId = Utilities::getValueInFilterDataSourceByName($dataSet, 'issue', true);
        $filter = Utilities::createQueryFiltered($dataSet);
        try {
            $task = $this->taskService->getTaskRelationshipByIssue($taskId[0],$filter);
            DevopsUtilities::setDataSetDefault($request, $response, $task);
        } catch (Exception $e) {
            $response->addMessage(new Response\Message('Não foi possível achar os relacionamentos de sua tarefa, pois ' . $e->getMessage()));
        }
    }

    public function removeKanbanItems(Request\DataSet $request, Response $response){
        $dataSet = $request->getDataSet()->getRows();
        $count = 0;
        foreach ($dataSet as $item) {
            try{
                $id = null;
                if(empty($item['id'])) {
                    $id = $item['ID'];
                }else{
                    $id = $item['id'];

                }
                $this->taskService->deleteIssues($id);

                $count++;
            }
            catch (\Exception $e){

                $response->addMessage(new Response\Message('Não foi possível remover a issue ' . $id . ', pois ' . $e->getMessage()));
            }
        }
        $response->addMessage(new Response\Message($count . ' issue(s) deletada(s) com sucesso'));
    }

    public function removeTaskRelationship(Request\DataSet $request, Response $response){
        $dataSet = $request->getDataSet()->getRows();
        foreach ($dataSet as $item) {
            try{
                $this->taskService->deleteTaskRelationship($item['ID'], $item['ISSUE']);
            }
            catch (\Exception $e){
                $response->addMessage(new Response\Message('Não foi possível deletar o relacionamento da tarefa ' . $item['ID'] . ' com a issue ' .
                                                            $item['ISSUE'] . ', pois ' . $e->getMessage()));
            }
        }
        $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_DELETED_ISSUE_RELATIONSHIP ));
    }

    public function saveTaskRelationship(Request\DataSet $request, Response $response){
        $tasks = $request->getDataSet()->getRows();
        $count = 0;
        foreach ($tasks as $task) {
            try{
                $this->taskService->createTaskRelationship($task['ID'], $task['ISSUE'],\Service\TaskPrecursor::RELATIONSHIP_ISSUE_TYPE);
                $this->taskService->applyOrderToIssueById($task['ISSUE'][0]);
                $count++;
            }
            catch(\Exception $e){
                $response->addMessage(new Response\Message(Messages::ERR_NOT_SUCCESSFUL_SAVED_EXPLANATION . $e->getMessage()));
            }
        }
        $response->addMessage(new Response\Message(count($tasks[0]['ISSUE']) . ' issue(s) relacionada(s) a ' . $count . ' tarefa(s)'));
    }

    /**
     * Gets all task options parameters of logged
     * organization.
     * @param Request\Filter $request
     * @param Response $response
     */
    public function findTaskOptionsParameters(Request\Filter $request, Response $response){
        $dataSet = $this->taskService->findTaskOptionsParameters();
        $response->addDataSet(new DataSet('taskOptionsParameter', $dataSet));
    }

    /**
     * Updates the task option parameter
     * @param Request\DataSet $request
     * @param Response $response
     * @return bool
     */
    public function updateTaskOptionsParameter(Request\DataSet $request, Response $response){
        $parameter = $request->getDataSet()->getRows();
        $parameter = $parameter[0];
        try{
            $propertyId             = $parameter['PROPERTY_ID'] ? $parameter['PROPERTY_ID'] : null;
            $initialTaskStatusId    = $parameter['INITIAL_TASK_STATUS_ID'] ? $parameter['INITIAL_TASK_STATUS_ID'] : null;
            $finalTaskStatusId      = $parameter['FINAL_TASK_STATUS_ID'] ? $parameter['FINAL_TASK_STATUS_ID'] : null;
            if(empty($propertyId)){
                $this->taskService->createTaskOptionsParameter($initialTaskStatusId, $finalTaskStatusId);
                $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_CREATED));
            } else {
                $this->taskService->updateTaskOptionsParameter($propertyId, $initialTaskStatusId, $finalTaskStatusId);
                $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_UPDATED));
            }
        } catch(\Exception $e){
            $response->addMessage(new Response\Message(Messages::ERR_NOT_SUCCESSFUL_SAVED_EXPLANATION.$e->getMessage()));
        }
    }
    public function viewCommentsSave(Request\DataSet $request, Response $response){
        $parameter = $request->getDataSet()->getRows();
        $parameter = $parameter[0];
        try{
            $issueId = $parameter['ISSUE'];
            if(empty($parameter['BASIC_WIDGET'])){
                $lastDate = $this->taskService->viewCommentsSaveData($issueId);
            }else{
                $lastDate = $this->taskService->viewCommentsSaveDataFist($issueId);
            }
            if($lastDate){
                $response->addDataset(new DataSet('lastDate', [$lastDate->format('Y-m-d H:i:s')]));
            }

        } catch(\Exception $e){
            $response->addMessage(new Response\Message(Messages::ERR_NOT_SUCCESSFUL_SAVED_EXPLANATION.$e->getMessage()));
        }
    }

    /**
     * @param Request\Filter $request
     * @param Response $response
     * @throws \Exception
     */
    public function appropriationValidationOfPreviousDay(Request\Filter $request, Response $response){
        try {
            $diffAppropriation = $this->taskService->appropriationValidationOfLast30Day();
            $response->addDataset(new DataSet('appropriationValidationOfPreviousDay',$diffAppropriation));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
            throw $e;
        }
    }
    public function appropriationValidation(Request\DataSet $request, Response $response){
       // validation in requestListener
    }

    public function findTasksAppropriatedForCurrentUserInRange(Request\Filter $request, Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $range = empty($conditions[0]) ? null : $conditions[0]['value'];
        $hideWeekend = empty($conditions[1]) ? null : $conditions[1]["value"];
        try{
            $tasks = $this->taskService->getTasksAppropriatedForCurrentUserInRange($range, $hideWeekend);
            $response->addDataset(new DataSet('findTasksAppropriatedForCurrentUserInRange', $tasks));
        }
        catch(\Exception $e){
            $response->addMessage(new Response\Message('Não foi possível achar suas tarefas apropriadas, pois ' . $e->getMessage()));
        }
    }

    public function findTasksNotAppropriatedForCurrentUser(Request\Filter $request, Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $range = empty($conditions[0]) ? null : $conditions[0]['value'];
        try {
            $tasks = $this->taskService->getTasksNotAppropriatedForCurrentUser($range);
            $response->addDataset(new DataSet('findTasksNotAppropriatedForCurrentUser', $tasks));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message('Não foi possível achar suas tarefas não apropriadas, pois ' . $e->getMessage()));
        }
    }

    public function findTasksForCurrentUser(Request\Filter $request, Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        try {
            $tasks = $this->taskService->getTasksForCurrentUser();
            $response->addDataset(new DataSet('findTasksForCurrentUser', $tasks));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message('Não foi possível achar suas tarefas não apropriadas, pois ' . $e->getMessage()));
        }
    }

    /***********************************************************************
     * Find task information for time control screen
     ***********************************************************************/
    public function findTaskForTimeControl(Request\Filter $request, Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $taskId = empty($conditions[0]) ? 0 : $conditions[0]['value'];
        try {
            $tasks = $this->taskService->getTaskForTimeControl($taskId);
            $response->addDataset(new DataSet('findTask', $tasks));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message('Não foi possível achar sua tarefa, pois' . $e->getMessage()));
        }
    }

    public function findTaskAllocationForCurrentUser(Request\Filter $request, Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $taskId = empty($conditions[0]) ? 0 : $conditions[0]['value'];
        try {
            $tasks = $this->taskService->getTaskAllocationForCurrentUser($taskId);
            $response->addDataset(new DataSet('findTaskAllocationForCurrentUser', $tasks));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message('Não foi possível achar a alocação do usuário logado com a tarefa dada, pois ' . $e->getMessage()));
        }
    }

    public function saveChangedFiles(Request\DataSet $request, Response $response){
        $parameter = $request->getDataSet()->getRows();
        $parameter = $parameter[0];
        $files = $parameter['FILES'];
        $task = $parameter['TASK'];
        $fileType = $parameter['FILE_TYPE'];
        $repository = isset($parameter['REPOSITORY']) && $parameter['REPOSITORY'] ? $parameter['REPOSITORY'] : null;
        try {
            $this->taskService->saveChangedFiles($files, $task, $fileType, $repository);
        } catch (Exception $e) {
            $response->addMessage(new Response\Message('Não foi possível salvar os arquivos modificados.'));
        }
    }

    public function bringChangedFilesByType(Request\DataSet $request, Response $response){
        $parameter    = $request->getDataSet()->getRows();
        $parameter    = $parameter[0];
        $files        = $parameter['FILES'];
        $repo         = $parameter['REPOSITORY'];
        $task         = $parameter['TASK'];
        $conflictOnly = isset($parameter['CONFLICT_ONLY']) ? $parameter['CONFLICT_ONLY'] : false;
        try {
            $files = $this->taskService->bringChangedFilesByType($files, $repo, $task, $conflictOnly);
            $response->addDataset(new DataSet('bringChangedFilesByType',$files));
        } catch (Exception $e) {
            $response->addMessage(new Response\Message('Não foi possível salvar os arquivos modificados.'));
        }
    }

    public function findMasterFiles(Request\DataSet $request, Response $response){
        $parameters = $request->getDataSet()->getRows();
        $param      = $parameters[0];
        $files      = $param['FILES'];
        $repo       = $param['REPOSITORY'];
        $task       = $param['TASK'];
        try {
            $masterFiles = $this->taskService->findMasterFiles($files, $repo, $task);
            $response -> addDataset(new DataSet('findMasterFiles', $masterFiles));
        } catch (Exception $e) {
            $response->addMessage(new Response\Message(Messages::ERR_MASTER_FILES_NOT_FOUND));
        }

    }

    public function isTaskDated(Request\Filter $request, Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $taskId = empty($conditions[0]) ? 0 : $conditions[0]['value'];
        try {
            $isDated = $this->taskService->isTaskProjectDated($taskId);
            $response->addDataset(new DataSet('isTaskDated', array(array('DATED' => $isDated))));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message('Não foi possível verificar se a tarefa é datada, pois ' . $e->getMessage()));
        }
    }

    public function findMyIssueLabels(Request\Filter $request, Response $response){
        try {
            $filter = $request->getFilterCriteria()->getConditions();
            $labels = $this->taskService->getMyIssueLabels($filter);
            $response->addDataset(new DataSet('findMyIssueLabels', $labels));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message('Não foi possível achar seus rótulos de issue, pois ' . $e->getMessage()));
        }
    }

    public function findMeetingLabels(Request\Filter $request, Response $response){
        try {
            $filter = $request->getFilterCriteria()->getConditions();
            $labels = $this->taskService->findMeetingLabels($filter);
            $response->addDataset(new DataSet('findLabelsMeeting', $labels));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message('Não foi possível achar seus rótulos de issue, pois ' . $e->getMessage()));
        }
    }

    public function findMeetingLabelsForSchedule(Request\Filter $request, Response $response){
        try {
            $filter = $request->getFilterCriteria()->getConditions();
            $projectId = Utilities::getValueInFilterDataSourceByName($filter,'PROJECT');
            $labels = $this->taskService->findMeetingLabelsForSchedule($projectId);
            $response->addDataset(new DataSet('findLabelsMeetingForSchedule', $labels));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message('Não foi possível achar seus rótulos de issue, pois ' . $e->getMessage()));
        }
    }

    public function saveIssueLabel(Request\DataSet $request, Response $response){
        try {
            $issueLabel = $request->getDataSet()->getRows();
            $issueLabel = $issueLabel[0];
            $id = empty($issueLabel['ID']) ? null : $issueLabel['ID'];
            $description = $issueLabel['DESCRIPTION'];
            $project = empty($issueLabel['PROJECT']) ? null : $issueLabel['PROJECT'];
            $dataSet = $this->taskService->persistIssueLabel($id, $description, $project);
            $response->addDataset(new DataSet('issueLabel', $dataSet));
            $response->addNotification(new Response\Notification('Rótulo de issue salvo com sucesso'));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message('Não foi possível salvar seu rótulo de issue, pois ' . $e->getMessage()));
        }
    }

    public function removeIssueLabels(Request\Row $request, Response $response){
        $issueLabel = $request->getRow();
        try{
            $this->taskService->removeIssueLabel($issueLabel['ID']);
        }
        catch(\Exception $e){
            $response->addNotification(new Response\Notification(new Response\Message($e->getMessage())));
        }
        $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_DELETED_ISSUE));
    }

	/************************************************************
	 * Gets all issues related to a specified task.
     * @param Request\Filter    $request
     * @param Response          $response
	 ************************************************************/
	public function findIssuesRelatedToTask(Request\Filter $request, Response $response){
		$filter = $request->getFilterCriteria()->getConditions();
		$taskId = DevOpsUtilities::getValueInFilterDataSourceByName($filter, 'TASK_ID', true);
		$conditions = DevOpsUtilities::createQueryFiltered($filter);
		$issues = $this->taskService->findIssuesRelatedToTask($taskId, $conditions);
		$response->addDataSet(new DataSet('issuesRelatedToTask', $issues));
	}

    /************************************************************
     * Gets all comments related to a specified issue.
     * @param Request\Filter    $request
     * @param Response          $response
     ************************************************************/
    public function findCommentsRelatedToIssue(Request\Filter $request, Response $response){
        $filter = $request->getFilterCriteria()->getConditions();
        $issueId = $filter[0]['value'][0];
        $comments = $this->taskService->findCommentsRelatedToIssue($issueId);
        $response->addDataSet(new DataSet('commentsRelatedToIssue', $comments));
    }

    /************************************************************
     * Gets all allocations to a specified task.
     * @param Request\Filter $request
     * @param Response $response
     ************************************************************/
    public function findStartedAllocationsByTask(Request\Filter $request, Response $response) {
        $filter = $request->getFilterCriteria()->getConditions();
        $task = $filter[0]['value'];
        $resource = $filter[1]['value'];
        $allocations = $this->taskService->getStartedAllocationsByTask($task, $resource);
        $response->addDataSet(new DataSet("startedAllocationsByTask", $allocations));
    }

    /****************************************************************************
     * Gets all generated tasks not concluded related to a specified task.
     * @param Request\Filter $request
     * @param Response $response
     ****************************************************************************/
    public function findTasksGeneratedNotConcluded(Request\Filter $request, Response $response){
        $filter = $request->getFilterCriteria()->getConditions();
        $task = $filter[0]['value'];
        $tasks = $this->taskService->findTasksGeneratedNotConcludedByTask($task);
        $response->addDataSet(new DataSet('tasksGeneratedNotConcluded', $tasks));
    }

    public function getProjectTasksInitialized(Request\Filter $request, Response $response)
    {
        $param = $request->getFilterCriteria()->getConditions();
        $projectId = $param[0]['value'];
        $tasksInitialized = $this->taskService->getProjectTasksInitialized($projectId);
        $response->addDataSet(new DataSet('tasksInitialized', $tasksInitialized));
    }

    /****************************************************************************
     * Gets issue description
     * @param Request\Filter $request
     * @param Response $response
     ****************************************************************************/
    public function findRelatedIssueDescription(Request\Filter $request, Response $response){
        $param          = $request->getFilterCriteria()->getConditions();
        $issueId        = $param[0]['value'][0];
        $description    = $this->taskService->getIssueDescription($issueId);
        $response->addDataSet(new DataSet('relatedIssueDescription', $description));
    }

    public function createTaskIssueMeetingSave(Request\DataSet $request, Response $response)
    {
        $rows = $request->getDataSet()->getRows();
        foreach ($rows as $row) {
            list($type, $name, $taskDescription, $followers, $projectId, $parent, $taskClassification, $resource, $productId,
                $sponsorId, $issueClassification, $reporter, $issueDescription, $scheduleProjectId, $scheduleParent,
                $scheduleClassification, $scheduleDescription, $scheduleResource, $labelsTask, $labelsIssue,
                $scheduleRelationshipIssueId, $scheduleRelationshipTaskId,$commitmentDateIssue,$commitmentDateTask,
                $orderIssue,$orderTask,$labelsSchedule, $taskStatusId, $priority) =
                $this->buildCreateTaskIssueMeeting($row);
            $reload = $this->taskService->createFastTaskAndIssue($type, $name, $projectId, $taskClassification, $resource, $parent,
                $taskDescription, $productId, $sponsorId, $followers, $issueClassification, $reporter, $issueDescription,
                $scheduleProjectId, $scheduleParent, $scheduleClassification, $scheduleDescription, $scheduleResource,
                $labelsTask, $labelsIssue, $scheduleRelationshipIssueId, $scheduleRelationshipTaskId,$commitmentDateIssue,
                $commitmentDateTask,$orderIssue,$orderTask,$labelsSchedule,$taskStatusId, $priority);
            $response->addDataSet(new DataSet('createTaskIssueMeeting', array('RELOAD'=>$reload)));
            $response->addNotification(new Response\Notification('Registro salvo com sucesso'));
        }
    }


    /**
     * @param $row
     * @return array
     */
    protected function getValuesCalculation($row)
    {
        $task = $row['task'];
        $begin = $row['begin'];
        $end = $row['end'];
        $duration = $row['duration'];
        $precursor = $row['precursor'];
        $precursorType = $row['precursorType'];
        $parent = $row['parent'];
        $exception = $row['exception'];
        $exceptionDate = $row['exceptionDate']?:null;
        $resourceIds = $row['resources'];
        $projectId = $row['project'];
        $latency = $row['latency'];


        $begin = \DateTime::createFromFormat(\Util\Utilities::FORMAT_DATE_DEFAULT,$begin.' 00:00:00');
        $end = \DateTime::createFromFormat(\Util\Utilities::FORMAT_DATE_DEFAULT,$end.' 00:00:00');

        return array($task,$begin, $end, $duration, $precursor, $precursorType, $parent, $exception, $exceptionDate,
            $projectId,$resourceIds,$latency);
    }

    public function duplicateChangedFilesForNewTask(Request\DataSet $request, Response $response){
        $params = $request->getDataSet()->getRows();
        $params = $params[0];
        $taskId = $params['TASK_ID'];
        $newTaskId = $params['NEW_TASK_ID'];
        $typeRelationship = $params['TYPE_RELATIONSHIP'];

        try{
            $this->taskService->duplicateChangedFilesForNewTask($taskId, $newTaskId, $typeRelationship);
        } catch(\Exception $e){
            $response->addMessage(new Response\Message($e->getMessage()));
        }
    }

    public function saveLabelTask(Request\DataSet $request, Response $response)
    {
        $issuesAndLabels = $request->getDataSet()->getRows()[0];
        $task = $issuesAndLabels['ID'];
        $labels = $issuesAndLabels['TASK_LABEL'];
        $taskResponse = $issuesAndLabels['TASK_RESPONSE'];
        try {
            $this->taskService->associateTaskAndLabels($task, $labels, null, true);
            $this->taskService->saveTaskReponse($task, $taskResponse);
            $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_CREATED));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage()));
        }
    }

    /**
     * @param $row
     * @return array
     */
    protected function buildCreateTaskIssueMeeting($row)
    {
        $type = $row['TYPE'];
        $name = $row['NAME'];
        $taskDescription = empty($row['TASK_DESCRIPTION']) ? null : $row['TASK_DESCRIPTION'];
        $issueDescription = empty($row['ISSUE_DESCRIPTION']) ? null : $row['ISSUE_DESCRIPTION'];
        $followers = empty($row['FOLLOWERS']) ? array() : $row['FOLLOWERS'];
        $scheduleProjectId = empty($row['SCHEDULE_PROJECT_ID']) ? null : $row['SCHEDULE_PROJECT_ID'];
        $scheduleParent = empty($row['SCHEDULE_PARENT']) ? null : $row['SCHEDULE_PARENT'];
        $scheduleClassification = empty($row['SCHEDULE_CLASSIFICATION_ID']) ? null : $row['SCHEDULE_CLASSIFICATION_ID'];
        $scheduleDescription = empty($row['SCHEDULE_DESCRIPTION']) ? null : $row['SCHEDULE_DESCRIPTION'];
        $projectId = empty($row['PROJECT_ID']) ? null : $row['PROJECT_ID'];
        $parent = empty($row['PARENT']) ? null : $row['PARENT'];
        $taskClassification = empty($row['TASK_CLASSIFICATION']) ? null : $row['TASK_CLASSIFICATION'];
        $productId = empty($row['PRODUCT_ID']) ? null : $row['PRODUCT_ID'];
        $sponsorId = empty($row['SPONSOR_ID']) ? null : $row['SPONSOR_ID'];
        $orderTask = empty($row['ORDER_TASK']) ? 0 : $row['ORDER_TASK'];
        $orderIssue = empty($row['ORDER_ISSUE']) ? 0 : $row['ORDER_ISSUE'];
        $issueClassification = empty($row['ISSUE_CLASSIFICATION']) ? null : $row['ISSUE_CLASSIFICATION'];
        $commitmentDateTask = empty($row['COMMITMENT_DATE_TASK']) ? null : \DateTime::createFromFormat(\Util\Utilities::FORMAT_DATE_DEFAULT,$row['COMMITMENT_DATE_TASK'].' 00:00:00');
        $commitmentDateIssue = empty($row['COMMITMENT_DATE_ISSUE']) ? null : \DateTime::createFromFormat(\Util\Utilities::FORMAT_DATE_DEFAULT,$row['COMMITMENT_DATE_ISSUE'].' 00:00:00');
        $reporter = empty($row['REPORTER']) ? null : $row['REPORTER'];
        $scheduleRelationshipIssueId = empty($row['SCHEDULE_RELATIONSHIP_ISSUE']) ? array() : $row['SCHEDULE_RELATIONSHIP_ISSUE'];
        $scheduleRelationshipTaskId = empty($row['SCHEDULE_RELATIONSHIP_TASK']) ? array() : $row['SCHEDULE_RELATIONSHIP_TASK'];
        $resource = empty($row['RESOURCE']) ? array() : $row['RESOURCE'];
        $scheduleResource = empty($row['SCHEDULE_RESOURCE']) ? array() : $row['SCHEDULE_RESOURCE'];
        $labelsTask = empty($row['LABELS_RELATED_USER_TASK']) ? array() : $row['LABELS_RELATED_USER_TASK'];
        $labelsIssue = empty($row['LABELS_RELATED_USER_ISSUE']) ? array() : $row['LABELS_RELATED_USER_ISSUE'];
        $labelsSchedule = empty($row['LABELS_RELATED_USER_SCHEDULE']) ? array() : $row['LABELS_RELATED_USER_SCHEDULE'];
        $taskStatusId = empty($row['TASK_STATUS']) ? null : $row['TASK_STATUS'];
        $priority       = empty($row["PRIORITY"]) ? null : $row["PRIORITY"];

        return array($type, $name, $taskDescription, $followers, $projectId, $parent, $taskClassification, $resource, $productId,
            $sponsorId, $issueClassification, $reporter, $issueDescription, $scheduleProjectId, $scheduleParent,
            $scheduleClassification, $scheduleDescription, $scheduleResource, $labelsTask, $labelsIssue,
            $scheduleRelationshipIssueId, $scheduleRelationshipTaskId,$commitmentDateIssue,$commitmentDateTask,
            $orderIssue,$orderTask,$labelsSchedule, $taskStatusId, $priority);
    }

    /**
     * @param Request\DataSet $request
     * @param Response $response
     * @throws \Exception
     */
    public function saveMultipleProjectTaskAndCreateRelationship(Request\DataSet $request, Response $response){
        $data = $request->getDataSet()->getRows();
        try{
            $param = $data[0];
            $tasks = $param['TASKS'];
            $isRelatedTask = empty($param['IS_RELATED_TASK']) ? true : $param['IS_RELATED_TASK'];
            $finishedTask = empty($param['FINISHED_TASK']) ? null : $param['FINISHED_TASK'];
            $document = empty($param['DOCUMENT']) ? null : $param['DOCUMENT'];
            $notSetTotalTime = empty($param['NOT_SET_TOTAL_TIME']) ? null : $param['NOT_SET_TOTAL_TIME'];
            $createdTaskIds = $this->taskService->saveMultipleProjectTask($tasks, $finishedTask, $isRelatedTask, $document,$notSetTotalTime);
            $response->addDataset(new DataSet("ID", $createdTaskIds));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * @param Request\DataSet $request
     * @param Response $response
     * @throws \Exception
     */
    public function generateTasksFromProjectTask(Request\DataSet $request, Response $response){
        $data = $request->getDataSet()->getRows();
        try{
            $param = $data[0];
            $tasks = $param['TASKS'];
            $classificationId = empty($param['CLASSIFICATION_ID']) ? true : $param['CLASSIFICATION_ID'];
            $versionDevId = empty($param['VERSION_DEV_ID']) ? null : $param['VERSION_DEV_ID'];
            $createdTaskIds = $this->taskService->saveMultipleProjectTaskFromProjectTask($tasks, $versionDevId);
            $response->addDataset(new DataSet("ID", $createdTaskIds));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }

    }

    public function getRelatedTasks(Request\Filter $request, Response $response){
        $filters = $request->getFilterCriteria()->getConditions();
        try{
            $tasks = DevOpsUtilities::getValueInFilterDataSourceByName($filters,'TASK');
            $relatedTasks = $this->taskService->findRelatedTasks($tasks);
            DevOpsUtilities::setDataSetDefault($request,$response,$relatedTasks);;
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function validateTaskToBeStarted(Request\Filter $request, Response $response)
    {
        try{
            $filters = $request->getFilterCriteria()->getConditions();
            $taskId = DevOpsUtilities::getValueInFilterDataSourceByName($filters, 'TASK_ID');
            $validateTaskToBeStarted = $this->taskService->validateTaskToBeStarted($taskId);
            DevOpsUtilities::setDataSetDefault($request, $response, $validateTaskToBeStarted);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getProjectsInfoCreateTasksByIssue(Request\Filter $request, Response $response)
    {
        try{
            $conditions = $request->getFilterCriteria()->getConditions();
            $getProjectsInfoCreateTasksByIssue = $this->taskService->getProjectsInfoCreateTasksByIssue($conditions);
            DevOpsUtilities::setDataSetDefault($request, $response, $getProjectsInfoCreateTasksByIssue);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function taskHasBranch(Request\Filter $request, Response $response)
    {
        try{
            $conditions = $request->getFilterCriteria()->getConditions();
            $tasks = Utilities::getValueInFilterDataSourceByName($conditions, 'TASKS');
            $targetProjectId = Utilities::getValueInFilterDataSourceByName($conditions, 'TARGET_PROJECT_ID');
            $projectId = Utilities::getValueInFilterDataSourceByName($conditions, 'PROJECT_ID');
            $tasksWithBranch = $this->taskService->taskHasBranch($tasks, $targetProjectId, $projectId);
            DevOpsUtilities::setDataSetDefault($request, $response, $tasksWithBranch);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }

    }

}
