<?php
namespace Service;

use Doctrine\ORM\EntityManager;
use Util\Instance\Repository as RepositoryInstance;
use Util\InstanceProvider;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Zeedhi\DevOps\Model\Projectmanagement\Repositories;
use \Util\Utilities;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;
use Util\Messages;
use \Factory;
use Zeedhi\Framework\DataSource\Operator\In;

class Issue
{
    /** @var Repositories\Project  */
    protected $projectRepository;

    /** @var Repositories\ResourceProject  */
    protected $resourceProjectRepository;
    /**
     * @var EntityManager $entityManager
     */
    private $entityManager;

    /**
     * @var \Util\Utilities $environment
     */
    private $environment;

    /**
     * @var Allocation $allocationService
     */
    private $allocationService;

    /**
     * @var \Doctrine\ORM\EntityRepository $allocationRepository
     */
    private $allocationRepository;

    /**
     * @var Repositories\Task $taskRepository
     */
    private $taskRepository;

    /**
     * @var Repositories\Label $labelRepository
     */
    private $labelRepository;
    /**
     * @var Repositories\CommentType $commentTypeRepository
     */
    private $commentTypeRepository;
    /**
     * @var Repositories\TaskType $taskTypeRepository
     */
    private $taskTypeRepository;
    /**
     * @var Repositories\TaskTypeClassification $taskTypeClassificationRepository
     */
    private $taskTypeClassificationRepository;
    /**
     * @var Repositories\Classification $classificationRepository
     */
    private $classificationRepository;
    /**
     * @var Repositories\TaskTypeCommentType $taskTypeCommentTypeRepository
     */
    private $taskTypeCommentTypeRepository;
    /**
     * @var Repositories\TaskTypeCommentTypeClassif $taskTypeCommentTypeClassifRepository
     */
    private $taskTypeCommentTypeClassifRepository;
    /**
     * @var Repositories\TaskRelationship
     */
    private $taskRelationshipRepository;
    /**
     * @var \Factory\Comments
     */
    private $commentsFactory;
    /**
     * @var \Factory\TaskTypeCommentType
     */
    private $taskTypeCommentTypeFactory;
    /**
     * @var \Factory\TaskTypeCommentTypeClassif
     */
    private $taskTypeCommentTypeClassifFactory;

    /** @var \Teknisa\Model\Teknisa\Repositories\CpCall $callRepository */
    public $callRepository;


    public function __construct(
        EntityManager $entityManager, \Util\Utilities $environment, Allocation $allocationService,
        Factory\TaskLabel $taskLabelFactory,\Factory\Comments $commentsFactory,
        \Factory\TaskTypeCommentType $taskTypeCommentTypeFactory,\Factory\TaskTypeCommentTypeClassif $taskTypeCommentTypeClassifFactory
    ){
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->allocationService = $allocationService;
        $this->allocationRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR.'Allocation');
        $this->taskRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR.'Task');
        $this->taskRelationshipRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR.'TaskRelationship');
        $this->taskStatusRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR.'TaskStatus');
        $this->taskTypeRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR.'TaskType');
        $this->taskTypeClassificationRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR.'TaskTypeClassification');
        $this->labelRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR.'Label');
        $this->resourceProjectRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR.'ResourceProject');
        $this->projectRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR.'Project');
        $this->commentTypeRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR.'CommentType');
        $this->classificationRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR.'Classification');
        $this->taskTypeCommentTypeRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR.'TaskTypeCommentType');
        $this->taskTypeCommentTypeClassifRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR.'TaskTypeCommentTypeClassif');
        $this->taskLabelFactory = $taskLabelFactory;
        $this->commentsFactory = $commentsFactory;
        $this->taskTypeCommentTypeFactory = $taskTypeCommentTypeFactory;
        $this->taskTypeCommentTypeClassifFactory = $taskTypeCommentTypeClassifFactory;
        $this->callRepository = $this->entityManager->getRepository('\Teknisa\Model\Teknisa\Entities\CpCall');
    }

    public function addFollower(Entities\Task $issue, Entities\Resource $resource){
        return $this->allocationService->createAllocation($issue, $resource, Allocation::FOLLOWER);
    }

    public function markAsRead($issueId){
        $userEntity  = $this->environment->getCurrentUser();
        /** @var Entities\Task $issueEntity */
        $issueEntity = $this->taskRepository->find($issueId);
        /** @var Entities\Allocation[] $allocationEntities */
        $allocationEntities = $this->allocationRepository->findBy(array('task' => $issueId, 'resource' => $userEntity));
        if(empty($allocationEntities)){
            $allocationEntities[] = $this->addFollower($issueEntity, $userEntity);
        }
        foreach ($allocationEntities as $allocationEntity) {
            $allocationEntity->setLastPostView($this->environment->getCurrentTime());
            $this->entityManager->persist($allocationEntity);
            $this->entityManager->flush($allocationEntity);
        }

    }
    /**
     * @return array
     */
    public function getAllMeetingProjectIds()
    {
        $projectType = Utilities::getProjectTypeMeeting();
        $meetingProject = $this->projectRepository->findBy(array('projectType' => $projectType));
        $projectIds = Utilities::getArrayByArrayObjectWithGet($meetingProject);
        return $projectIds;
    }
    /**
     * @return array
     */
    public function getMyMeetingProjectIds()
    {
        $projectType = Utilities::getProjectTypeMeeting();
        /** @var Entities\ResourceProject[] $projectResources */
        $meetingProject = $this->projectRepository->findBy(array('projectType' => $projectType));;
        $projectResources = $this->resourceProjectRepository->findBy(array('type' => Entities\ResourceProject::MANAGER,
                            'resourceC' => $this->environment->getCurrentUserId(),'project' => $meetingProject));
        $projectIds = Utilities::getArrayByArrayObjectWithGet($projectResources,array('getProject','getId'));

        return $projectIds;
    }
    /**
     * @return array
     */
    public function getImInMeetingProjectIds()
    {
        $projectType = Utilities::getProjectTypeMeeting();
        /** @var Entities\ResourceProject[] $projectResources */
        $meetingProject = $this->projectRepository->findBy(array('projectType' => $projectType));;
        $projectResources = $this->resourceProjectRepository->findBy(array(
            'resourceC' => $this->environment->getCurrentUserId(),
            'project' => $meetingProject));
        $projectIds = Utilities::getArrayByArrayObjectWithGet($projectResources,array('getProject','getId'));

        return $projectIds;
    }

    public function getLabelsByOperatorId($filter = array()){
        $projectIds = $this->getImInMeetingProjectIds();
        $filterString = DevOpsUtilities::createQueryFiltered($filter);
        return $this->labelRepository->getLabelsByOperatorId($this->environment->getCurrentUserId(),$projectIds,$filterString);
    }

    public function validateRemoveTaskType(Entities\TaskType $taskType){

        $taskStatus = $this->taskStatusRepository->findOneBy(array('taskType'=>$taskType));
        $message = '';
        if(!empty($taskStatus)){
            $message = Messages::ERR_NOT_POSSIBLE_TO_DELETE_ISSUE_STATUS_ALREADY_USED;
            $comma = '<br/>';
            /** @var Entities\TaskStatus[] $tasksStatus */
            $tasksStatus = $this->taskStatusRepository->findBy(array('taskType' => $taskType->getId()));
            foreach($tasksStatus as $taskStatus){
                $message .= $comma."\"".$taskStatus->getLabel()."\"";
                $comma  = ', ';
            }
        }

        if($message){
            throw new \Exception($message);
        }

    }

    public function removeTaskType($id){
        /** @var Entities\TaskType $taskType */
        $taskType = $this->taskTypeRepository->find($id);
        $this->validateRemoveTaskType($taskType);
        /** @var Entities\TaskTypeClassification[] $taskTypeClassifications */
        $taskTypeClassifications = $this->taskTypeClassificationRepository->findBy(array('taskType' => $taskType));
        foreach ($taskTypeClassifications as $taskTypeClassification) {
            $this->entityManager->remove($taskTypeClassification);
            $this->entityManager->flush();
        }

        /** @var Entities\TaskTypeCommentType[] $taskTypeCommentTypes */
        $taskTypeCommentTypes = $this->taskTypeCommentTypeRepository->findBy(array('taskType' => $taskType));
        foreach ($taskTypeCommentTypes as $taskTypeCommentType) {
            /** @var Entities\TaskTypeCommentTypeClassif[] $taskTypeCommentTypeClassifs */
            $taskTypeCommentTypeClassifs = $this->taskTypeCommentTypeClassifRepository->findBy(array('taskTypeCommentType' => $taskTypeCommentType));
            foreach ($taskTypeCommentTypeClassifs as $taskTypeCommentTypeClassif) {
                $this->entityManager->remove($taskTypeCommentTypeClassif);
                $this->entityManager->flush();
            }
            $this->entityManager->remove($taskTypeCommentType);
            $this->entityManager->flush();
        }

        $this->entityManager->remove($taskType);
        $this->entityManager->flush();

    }

    public function saveIssueOrderToFrame($taskTypeId,$orderToFrame,$allowPlayTaskInFrame){

        /** @var Entities\TaskType $taskType */
        $taskType = $this->taskTypeRepository->find($taskTypeId);
        $taskType->setOrderToFrame($orderToFrame);
        $taskType->setAllowPlayTaskInFrame($allowPlayTaskInFrame);
        $this->entityManager->persist($taskType);
        $this->entityManager->flush();
    }

    public function removeTaskTypeClassification($taskTypeClassificationId){

        /** @var Entities\TaskTypeClassification $taskTypeClassification */
        $taskTypeClassification = $this->taskTypeClassificationRepository->find($taskTypeClassificationId);
        $this->entityManager->remove($taskTypeClassification);
        $this->entityManager->flush();
    }
    public function removeOutputCommentIssueStatus($outputCommentId){

        /** @var Entities\TaskTypeCommentType $taskTypeCommentType */
        $taskTypeCommentType = $this->taskTypeCommentTypeRepository->findOneBy(array('id' => $outputCommentId));
        /** @var Entities\TaskTypeCommentTypeClassif[] $taskTypeCommentTypeClassifs */
        $taskTypeCommentTypeClassifs = $this->taskTypeCommentTypeClassifRepository->findBy(array('taskTypeCommentType' => $taskTypeCommentType));
        foreach ($taskTypeCommentTypeClassifs as $taskTypeCommentTypeClassif) {
            $this->entityManager->remove($taskTypeCommentTypeClassif);
            $this->entityManager->flush();
        }
        $this->entityManager->remove($taskTypeCommentType);
        $this->entityManager->flush();
    }

    public function saveAddTaskClassification($issueClassifications,$taskTypeId){

        $classifications = [];
        $taskType = $this->taskTypeRepository->find($taskTypeId);
        $taskService = Utilities::getTaskService();
        foreach ($issueClassifications as $issueClassification) {
            $classifications[] = $issueClassification['ID'];
        }

        $taskService->saveTaskTypeClassification($classifications,$taskType,Entities\TaskTypeClassification::TYPE_LOCKED,false);
        $this->entityManager->flush();
    }

    public function saveAddProjectType($projectTypes,$taskTypeId){

        $projectTypeIds = [];
        $taskType = $this->taskTypeRepository->find($taskTypeId);
        $taskService = Utilities::getTaskService();
        foreach ($projectTypes as $projectType) {
            $projectTypeIds[] = $projectType['ID'];
        }

        $taskService->saveTaskTypeProjectType($projectTypeIds,$taskType);
        $this->entityManager->flush();
    }

    public function removeCurrentUserFollower($issueId){

        $resourceId = InstanceProvider::getEnvironment()->getCurrentResourceId();
        $allocationRepository = InstanceProvider::getRepository()->getAllocation();
        $entityManager = InstanceProvider::getEntityManager();
        $filter = array('task' => $issueId, 'type' => Entities\Allocation::TYPE_FOLLOWER,'resource' => $resourceId);
        $allocationFollower = $allocationRepository->findOneBy($filter);
        $entityManager->remove($allocationFollower);
        $entityManager->flush();
    }

    public function sendOutComment($issueId,$document,$taskTypeTargetId,$commentTypeId,$description){
        /** @var Entities\Task $issue */
        $issue = $this->taskRepository->find($issueId);
        /** @var Entities\TaskType $taskTypeTarget */
        $taskTypeTarget = $this->taskTypeRepository->find($taskTypeTargetId);
        $taskStatus = $taskTypeTarget->getInitialTaskStatus();
        if(!$taskStatus){
            $taskStatus = $this->taskStatusRepository->findOneBy(array('TASK_TYPE' => $taskTypeTargetId));
        }
        $issue->setTaskStatus($taskStatus);
        $issue->setCurrentTaskStatusIn(new \DateTime());
        $this->entityManager->persist($issue);
        /** @var Entities\CommentType $commentType */
        $commentType = $this->commentTypeRepository->find($commentTypeId);
        $comment = $this->commentsFactory->createComments($issue, $description,$commentType);
        $this->entityManager->persist($comment);
        $this->entityManager->flush();
        if($document){
            Utilities::getAttachmentService()->addDocumentComment($comment->getId(), $document);
        }
        if($taskStatus->getControl() == 'H'){
            Utilities::getTaskService()->sendMailOnAvailableIssue($issue->getId(), $description);
        }else{
           Utilities::getCommentService()->sendsMailAdvisingAboutNewIssueComment($issue->getId(), $description);
        }

    }

    public function saveOutputCommentIssueStatus($outputCommentId,$action,$issueClassificationIds,$description,
                                                 $commentTypeId,$targetStatusId,$taskTypeId){

        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            /** @var Entities\CommentType $commentType */
            $commentType = $this->commentTypeRepository->find($commentTypeId);


            /** @var Entities\TaskType $taskType */
            $taskType = $this->taskTypeRepository->find($taskTypeId);

            /** @var Entities\TaskType $targetStatus */
            $targetStatus = $this->taskTypeRepository->find($targetStatusId);

            /** @var Entities\TaskTypeCommentType $taskTypeCommentType */
            $taskTypeCommentType = $this->taskTypeCommentTypeRepository->findOneBy(array('id' => $outputCommentId));
            $taskTypeCommentType = $this->taskTypeCommentTypeFactory->createTaskTypeCommentType($taskTypeCommentType, $taskType, $commentType);

            $this->entityManager->persist($taskTypeCommentType);
            $this->entityManager->flush();

            $this->setIssueClassifications($issueClassificationIds, $taskTypeCommentType);
            $taskTypeCommentType->setCommentDescription($description);
            $taskTypeCommentType->setInOutComment(Entities\TaskTypeCommentType::OUT_COMMENT);
            $taskTypeCommentType->setTaskTypeTarget($targetStatus);
            $taskTypeCommentType->setActionLabel($action);

            $this->entityManager->persist($taskTypeCommentType);
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
        return $taskTypeCommentType;
    }

    public function saveEntranceCommentIssueStatus($taskTypeId,$issueClassificationIds,$description,$commentTypeId){

        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();

            /** @var Entities\TaskType $taskType */
            $taskType = $this->taskTypeRepository->find($taskTypeId);
            if($commentTypeId){
                /** @var Entities\CommentType $commentType */
                $commentType = $this->commentTypeRepository->find($commentTypeId);

                /** @var Entities\TaskTypeCommentType $taskTypeCommentType */
                $taskTypeCommentType = $this->taskTypeCommentTypeRepository->findOneBy(array('taskType' => $taskType, 'inOutComment' => Entities\TaskTypeCommentType::IN_COMMENT));
                if($commentType){
                    $taskTypeCommentType = $this->taskTypeCommentTypeFactory->createTaskTypeCommentType($taskTypeCommentType, $taskType, $commentType);
                }

                $this->entityManager->persist($taskTypeCommentType);
                $this->entityManager->flush();
                $this->setIssueClassifications($issueClassificationIds, $taskTypeCommentType);
                $taskTypeCommentType->setCommentDescription($description);
                $taskTypeCommentType->setInOutComment(Entities\TaskTypeCommentType::IN_COMMENT);
                $this->entityManager->persist($taskTypeCommentType);
                $this->entityManager->flush();
            }else{
                $taskTypeCommentType = $this->taskTypeCommentTypeRepository->findOneBy(array('taskType' => $taskType, 'inOutComment' => Entities\TaskTypeCommentType::IN_COMMENT));
                /** @var Entities\TaskTypeCommentTypeClassif[] $taskTypeCommentTypeClassifs */
                $taskTypeCommentTypeClassifs = $this->taskTypeCommentTypeClassifRepository->findBy(array('taskTypeCommentType' => $taskTypeCommentType));
                foreach ($taskTypeCommentTypeClassifs as $taskTypeCommentTypeClassif) {
                        $this->entityManager->remove($taskTypeCommentTypeClassif);
                        $this->entityManager->flush();
                }
                $this->entityManager->remove($taskTypeCommentType);
                $this->entityManager->flush();
            }
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function framingAutomaticIssue()
    {
        $environment = Utilities::getEnvironment();
        $environment->setCurrentUserId(Utilities::getCurrentUserIdPm());
        $today = new \DateTime();
        $allIssueNeedFramingAutomatic = $this->taskRepository->getAllIssueNeedFramingAutomatic();
        foreach ($allIssueNeedFramingAutomatic as $issueNeedFramingAutomatic) {
            $connection = $this->entityManager->getConnection();
            try {
                $connection->beginTransaction();
                /** @var Entities\Task $issue */
                $issue = $this->taskRepository->find($issueNeedFramingAutomatic['ID']);
                /** @var Entities\TaskStatus $taskStatus */
                $taskStatus = $this->taskStatusRepository->find($issueNeedFramingAutomatic['TASK_STATUS_AUTO_CHANGE_STATUS']);
                $organization = $issueNeedFramingAutomatic['ORGANIZATION'];
                $environment->setOrganizationId($organization);
                $descriptionAutoChangeStatus = $issueNeedFramingAutomatic['DESCRIPTION_AUTO_CHANGE_STATUS'];
                $this->entityManager->persist($issue);
                $this->entityManager->flush();
                /** @var Entities\CommentType $commentType */
                $commentType = $this->commentTypeRepository->findOneBy(
                    array('control' => Entities\CommentType::TYPE_FRAMING_AUTOMATIC,
                        'organizationId' => $organization));
                if(empty($descriptionAutoChangeStatus)){
                    $descriptionAutoChangeStatus = $commentType->getCommentDescription();
                }

                $comment = $this->commentsFactory->createComments($issue, $descriptionAutoChangeStatus);
                $comment->setCommentType($commentType);
                $issue->setTaskStatus($taskStatus);
                $issue->setCurrentTaskStatusIn($today);
                $this->entityManager->persist($comment);
                $this->entityManager->flush();
                $connection->commit();
                Utilities::getCommentService()->sendsMailAdvisingAboutNewIssueComment($issue->getId(), $descriptionAutoChangeStatus);
                echo "\n----------------------------------------\n" .
                    'Issue com id ' . $issue->getId() . ' com o nome "' . $issue->getName() .
                    '" para o status com id ' . $taskStatus->getId() . ' com o nome "' . $taskStatus->getLabel() .
                    '" com o comentario "' . $descriptionAutoChangeStatus .
                    '" da organizacao do id ' . $organization .
                    "\n----------------------------------------\n";
            } catch (\Exception $e) {
                echo $e->getMessage();
                try {
                    $connection->rollBack();
                } catch (\Exception $e) {
                    echo $e->getMessage();
                }
            }
        }
        echo 'Terminado';

    }

    public function markManyAsRead($issuesIds){
        foreach($issuesIds as $id){
            $this->markAsRead($id);
        }
    }

    public function validateEntranceComment($issueId,$typeTask){
       return $this->taskRepository->validateEntranceComment($issueId,$typeTask);
    }

    public function getClientAcceptTask(Entities\Task $issue){
        $tasks = $issue->getTaskForIssue();
        foreach ($tasks as $task) {
            if($task->getPercentCompleted() != 100 &&
                $task->getClassification() &&
                $task->getClassification()->getControl() == Entities\Classification::CONTROL_CLIENT_ACCEPT )
            {
                    return $task;
            }
        }
        return null;
    }

    public function applyOrderToIssueById($issueId){
        Utilities::getTaskService()->applyOrderToIssueById($issueId);
    }

    public function saveGiveUpImplementationIssue($issueId)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            /** @var Entities\Task $issue */
            $issue = $this->taskRepository->find($issueId);
            $tasksFromIssue = $this->taskRepository->getTasksFromIssue($issueId);
            foreach ($tasksFromIssue as $taskFromIssue) {
                /** @var Entities\Task $taskFromIssue */
                $taskFromIssue = $this->taskRepository->find($taskFromIssue['ID']);
                $taskFromIssue->setStatus(Entities\Task::NOT_ACTIVE);
                $this->entityManager->persist($taskFromIssue);
            }
            $task = $this->getClientAcceptTask($issue);
            if($task){
                $task->setPercentCompleted(100);
                InstanceProvider::getService()->getTask()->updateTaskAndAllocationStatus($task, Utilities::getTaskStatusIsConcluded());
                $this->entityManager->persist($task);
            }
            $taskStatusEndId = InstanceProvider::getService()->getTask()->findTaskOptionsParameters()[0]['FINAL_TASK_STATUS_ID'];
            /** @var Entities\TaskStatus $taskStatusEnd */
            $taskStatusEnd = RepositoryInstance::getTaskStatus()->find($taskStatusEndId);
            $issue->setTaskStatus($taskStatusEnd);
            $this->entityManager->persist($issue);
            $this->entityManager->flush();
            Integration\IntegrationSM::integrationSMCancel($issueId, $issue->getOrganization());

            $connection->commit();
            return $issue;
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function clientAccept($issueId, $commentTypeId, $description, $document, $validateReporter = true)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            /** @var Entities\Task $issue */
            $issue = $this->taskRepository->find($issueId);
            if ($validateReporter) {
                $user = $this->environment->getCurrentUser();
                /** @var Entities\Allocation $reporter */
                $reporter = $this->allocationRepository->findOneBy(array('type' => \Service\Allocation::REPORTER, 'resource' => $user, 'task' => $issue));
                if (empty($reporter)) {
                    throw new \Exception(Messages::ERR_WRONG_REQUESTER_ACCEPT);
                }
            }

            $tasksFromIssue = $this->taskRepository->getTasksFromIssue($issueId);
            foreach ($tasksFromIssue as $taskFromIssue) {
                $taskFromIssue = $this->taskRepository->find($taskFromIssue['ID']);
                if ($taskFromIssue->getPercentCompleted() < 100) {
                    $taskFromIssue->setUsedInFraming('Y');
                }
                $this->entityManager->persist($taskFromIssue);
            }
            $task = $this->getClientAcceptTask($issue);
            $task->setPercentCompleted(100);
            InstanceProvider::getService()->getTask()->updateTaskAndAllocationStatus($task, Utilities::getTaskStatusIsConcluded());
            $commentType = $this->commentTypeRepository->find($commentTypeId);
            $comment = $this->commentsFactory->createComments($issue, $description, $commentType);
            $issue->setCurrentTaskStatusIn(Utilities::getEnvironment()->getCurrentTime());
            $this->entityManager->persist($issue);
            $this->entityManager->persist($task);
            $this->entityManager->persist($comment);
            $this->entityManager->flush();
            Integration\IntegrationSM::integrationSMAccept($issueId, $issue->getOrganization());
            if ($document) {
                Utilities::getAttachmentService()->addDocumentComment($comment->getId(), $document);
            }
            Utilities::getCommentService()->sendsMailAdvisingAboutNewIssueComment($issue->getId(), $description);
            $this->applyOrderToIssueById($issue->getId());

            $connection->commit();
            return $issue;
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function linkDownloadBudgetSm($issueId){
        $linkDownloadBudgetSm = array();
        $budget = RepositoryInstance::getBudget()->findOneBy(array('task' => $issueId));
        if($budget && $budget->getExternalId()){
            $organizationId = $this->environment->getCurrentOrganizationId();
            $budgetRepository = InstanceProvider::getRepository()->getBudget();
            $urlSm = $budgetRepository->getUrlIntegrationSM($organizationId);
            if($urlSm){
                $linkDownloadBudgetSm = Integration\IntegrationSM::linkDownloadBudgetSm($issueId,$organizationId);
            }
        }
        return $linkDownloadBudgetSm;
    }

    public function clientRefuse($issueId, $commentTypeId, $description, $document, $validateReporter = true)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            /** @var Entities\Task $issue */
            $issue = $this->taskRepository->find($issueId);
            if ($validateReporter) {
                $user = $this->environment->getCurrentUser();
                /** @var Entities\Allocation $reporter */
                $reporter = $this->allocationRepository->findOneBy(array('type' => \Service\Allocation::REPORTER, 'resource' => $user, 'task' => $issue));
                if (empty($reporter)) {
                    throw new \Exception(Messages::ERR_WRONG_REQUESTER_REFUSE);
                }
            }
            $tasksFromIssue = $this->taskRepository->getTasksFromIssue($issueId);
            foreach ($tasksFromIssue as $taskFromIssue) {
                $taskFromIssue = $this->taskRepository->find($taskFromIssue['ID']);
                $taskFromIssue->setUsedInFraming('N');
                $this->entityManager->persist($taskFromIssue);
            }
            $task = $this->getClientAcceptTask($issue);
            $task->setPercentCompleted(100);
            InstanceProvider::getService()->getTask()->updateTaskAndAllocationStatus($task, Utilities::getTaskStatusIsConcluded());
            $commentType = $this->commentTypeRepository->find($commentTypeId);
            $comment = $this->commentsFactory->createComments($issue, $description, $commentType);
            $issue->setCurrentTaskStatusIn(Utilities::getEnvironment()->getCurrentTime());
            $taskStatus = $this->taskStatusRepository->findOneBy(array('control' => Entities\TaskStatus::TASK_STATUS_CONTROL_BACKLOG));
            $issue->setTaskStatus($taskStatus);
            $this->entityManager->persist($issue);
            $this->entityManager->persist($task);
            $this->entityManager->persist($comment);
            $this->entityManager->flush();

            Integration\IntegrationSM::integrationSMCancel($issueId, $issue->getOrganization());
            /** @var Entities\Budget $budget */
            $budget = RepositoryInstance::getBudget()->findOneBy(array('task' => $issueId));
            if($budget){
                $budget->setExternalId(null);
                $this->entityManager->persist($budget);
                $this->entityManager->flush();
            }

            if ($document) {
                Utilities::getAttachmentService()->addDocumentComment($comment->getId(), $document);
            }
            Utilities::getCommentService()->sendsMailAdvisingAboutNewIssueComment($issue->getId(), $description);
            $this->applyOrderToIssueById($issue->getId());

            $connection->commit();
            return $issue;
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function validateSpecificationRequirements(){
        return $this->taskRepository->validateSpecificationRequirements($this->environment->getCurrentOrganizationId());
    }

    public function validateFinishSpecificationTask($taskId){
        /** @var Entities\TaskRelationship[] $issueRelationships */
        $issueRelationships = $this->taskRelationshipRepository->findBy(
            array('taskRelationship' => $taskId,'type' => Entities\TaskRelationship::RELATIONSHIP_ISSUE_TYPE)
        );
        $budgetRepository = InstanceProvider::getRepository()->getBudget();
        $organizationId = $this->environment->getCurrentOrganizationId();
        $urlSm = $budgetRepository->getUrlIntegrationSM($organizationId);
        if($urlSm){
            foreach ($issueRelationships as $issueRelationship) {
                $issueId = $issueRelationship->getTask()->getId();
                if($budgetRepository->validateBudget($issueId)){
                    Integration\IntegrationSM::validateSpecification($issueId);
                }
            }
        }
        return $this->taskRepository->validateFinishSpecificationTask($taskId);
    }

    public function associateIssuesAndLabels($issuesIds, $labelsIds){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();

        try {
            $labelsEntities = $this->getLabelsEntitiesByLabelsIds($labelsIds);
            foreach($issuesIds as $issueId){
                $issueEntity = $this->entityManager->getRepository(Utilities::ENTITY_DIR.'Task')->find($issueId);
                if($issueEntity){
                    foreach($labelsEntities as $labelEntity){
                        if( ! $this->isLabelEntityAlreadyAssociated($issueId, $labelEntity)){
                            $taskLabelEntity = $this->addLabel($issueEntity, $labelEntity);
                            $this->entityManager->persist($taskLabelEntity);
                        }
                    }
                }
            }
            $this->entityManager->flush();
            $connection->commit();
        } catch(\Exception $e){
            $connection->rollBack();
            throw $e;
        }

        return true;
    }

    private function isLabelEntityAlreadyAssociated($taskId, Entities\Label $labelEntity){
        $taskLabelEntity = $this->entityManager->getRepository(Utilities::ENTITY_DIR.'TaskLabel')
            ->findOneBy(array('task' => $taskId, 'label' => $labelEntity));
        return !empty($taskLabelEntity);
    }

    public function findIssueOrderToFrame($issueStatusId){
        return  $this->taskTypeRepository->findIssueOrderToFrame($issueStatusId);
    }

    public function findTaskTypesForFraming(Entities\Task $issue){
        return $this->taskTypeClassificationRepository->findTaskTypesForFraming($issue->getId());
    }

    public function findCommentTypeIssueStatus($conditions = null){
        $filterString = Utilities::createQueryFiltered($conditions);
        $organizationId = Utilities::getEnvironment()->getCurrentOrganizationId();
        return  $this->commentTypeRepository->findCommentTypeIssueStatus($organizationId, $filterString);
    }

    public function findEntranceCommentIssueStatus($taskTypeId){
        return  $this->taskTypeRepository->findEntranceCommentIssueStatus($taskTypeId);
    }

    public function findOutputCommentIssueStatus($taskTypeId){
        return  $this->taskTypeRepository->findOutputCommentIssueStatus($taskTypeId);
    }

    public function findIssueFramingClassification($issueStatusId){
        return  $this->taskTypeRepository->findIssueFramingClassification($issueStatusId);
    }

    public function findIssueFramingProjectType($issueStatusId){
        return  InstanceProvider::getRepository()->getProjectType()->findIssueFramingProjectType($issueStatusId);
    }

    public function removeIssueFramingProjectType($taskTypeProjectTypeId){
        $taskTypeProjectTypeRepository = InstanceProvider::getRepository()->getTaskTypeProjectType();
        $taskTypeProjectType = $taskTypeProjectTypeRepository->findOneBy(array('id' => $taskTypeProjectTypeId));
        $this->entityManager->remove($taskTypeProjectType);
        $this->entityManager->flush();
    }

    public function taskClassificationByIssue($issueId){
        return  $this->taskTypeRepository->taskClassificationByIssue($issueId);
    }

    public function taskClassificationByIssues($issueId){
        return  $this->taskTypeRepository->taskClassificationByIssues($issueId);
    }

    private function getLabelsEntitiesByLabelsIds($labelsIds){
        $labelsEntities = array();
        foreach($labelsIds as $labelId){
            array_push($labelsEntities, $this->entityManager->getRepository(Utilities::ENTITY_DIR.'Label')->find($labelId));
        }
        return $labelsEntities;
    }

    public function getIssueRelationship($issueId, $filter = ' '){
        return $this->taskRepository->getRelationedIssues($issueId, $filter);
    }

    public function getBudgetProduct($issueId){
        return $this->taskRepository->getBudgetProduct($issueId);
    }

     public function getBudgetProductByTask($taskId){
         $issues = InstanceProvider::getRepository()->getTask()->findIssuesRelatedToTask($taskId, $this->environment->getCurrentOrganizationId(), null);
        $issueProduct = $this->taskRepository->getBudgetProduct($issues[0]['ID']);
        $issueProduct['ISSUE_ID'] = $issues[0]['ID'];
        return $issueProduct;
    }

     public function saveProductBudget($productId, $issueId){
         $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
           // $callRepository = InstanceProvider::getEntityManager()->getRepository(Utilities::TEKNISA_ENTITY_DIR . 'CpCall');
            $callEntity = $this->callRepository->findOneBy(array('callRelationship' => $issueId));
            $callEntity->setProduct($productId);
            $this->entityManager->flush();
            $connection->commit($callEntity);
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function addLabel(Entities\Task $issueEntity, Entities\Label $labelEntity){
        return $this->taskLabelFactory->createTaskLabel($issueEntity, $labelEntity);
    }



    public function saveIssuesPriority($priority, $issues) {
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $issueIds = array();
            if($priority){
                $priority = $this->entityManager->find(Utilities::ENTITY_DIR . 'Priority', $priority);
            }else{
                $priority = null;
            }
            foreach($issues as $key => $issueId){
                $issuesIds[$key] = $issueId['ID'];
            }
            $issuesEntities = $this->taskRepository->findBy(array('id' => $issuesIds));

            foreach($issuesEntities as $issueEntity) {
                $issueEntity = InstanceProvider::getFactory()->getTask()->updatePriority($issueEntity, $priority);
                $this->entityManager->persist($issueEntity);
            }
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * @param $issueClassificationIds
     * @param $taskTypeCommentType
     */
    public function setIssueClassifications($issueClassificationIds,  $taskTypeCommentType)
    {
        $flush = false;
        if(empty($issueClassificationIds)){
            $taskTypeCommentTypeClassifs = $this->taskTypeCommentTypeClassifRepository->findBy(array('taskTypeCommentType' => $taskTypeCommentType));
            foreach ($taskTypeCommentTypeClassifs as $taskTypeCommentTypeClassif) {
                $this->entityManager->remove($taskTypeCommentTypeClassif);
                $this->entityManager->flush();
            }
        }else{
            /** @var Entities\Classification[] $issueClassifications */
            $issueClassifications = $this->classificationRepository->findBy(array('id' => $issueClassificationIds));
            foreach ($issueClassifications as $issueClassification) {
                $taskTypeCommentTypeClassif = $this->taskTypeCommentTypeClassifRepository->findOneBy(array('classification' => $issueClassification, 'taskTypeCommentType' => $taskTypeCommentType));
                if (!$taskTypeCommentTypeClassif) {
                    $taskTypeCommentTypeClassif = $this->taskTypeCommentTypeClassifFactory->createTaskTypeCommentTypeClassif($issueClassification, $taskTypeCommentType);
                    $this->entityManager->persist($taskTypeCommentTypeClassif);
                    $flush = true;
                }
            }
            if ($flush) {
                $this->entityManager->flush();
            }
            /** @var Entities\TaskTypeCommentTypeClassif[] $taskTypeCommentTypeClassifs */
            $taskTypeCommentTypeClassifs = $this->taskTypeCommentTypeClassifRepository->findBy(array('taskTypeCommentType' => $taskTypeCommentType));
            foreach ($taskTypeCommentTypeClassifs as $taskTypeCommentTypeClassif) {
                if (!in_array($taskTypeCommentTypeClassif->getClassification()->getId(), $issueClassificationIds)) {
                    $this->entityManager->remove($taskTypeCommentTypeClassif);
                    $this->entityManager->flush();
                }
            }
        }
    }
}
