<?php


namespace Service;


use Doctrine\ORM\EntityManager;
use Util\InstanceProvider;
use Util\Messages;
use Util\Utilities;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Zeedhi\DevOps\Model\Projectmanagement\Repositories;

class ProjectType
{

    const IS_ALLOW_END_TASK = Entities\ProjectType::IS_ALLOW_END_TASK;
    const IS_NOT_ALLOW_END_TASK = Entities\ProjectType::IS_NOT_ALLOW_END_TASK;
    protected $entityManager;
    protected $environment;
    /** @var Repositories\ProjectApproval */
    protected $projectApprovalRepository;
    /** @var Repositories\ProjectType */
    protected $projectTypeRepository;
    /** @var array */
    protected $uniqueControls;
    /** @var Repositories\Project */
    protected $projectRepository;
    /** @var Repositories\ClassificationProjectType */
    protected $classificationProjectTypeRepository;
    /**@var \Factory\ProjectApproval */
    private $projectApprovalFactory;
    /** @var \Factory\ProjectType */
    private $projectTypeFactory;


    public function __construct(EntityManager $entityManager, Utilities $environment,
                                \Factory\ProjectApproval $projectApprovalFactory,\Factory\ProjectType $projectTypeFactory)
    {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->projectApprovalFactory = $projectApprovalFactory;
        $this->projectApprovalRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'ProjectApproval');
        $this->projectTypeRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'ProjectType');
        $this->classificationProjectTypeRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'ClassificationProjectType');
        $this->projectRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'Project');
        $this->uniqueControls = array(Entities\ProjectType::TYPE_PROJECT_CONTROL_MEETING,
            Entities\ProjectType::TYPE_PROJECT_CONTROL_MEETING_OCCURRENCES);
        $this->projectTypeFactory = $projectTypeFactory;
    }


    public function setProfilesInProjectType(Entities\ProjectType $projectType, $profileExternalIds,$requiresApproval,
                                             $type = Entities\ProjectApproval::TYPE_AUTHORIZE)
    {
        if($requiresApproval){
            $this->createProjectApproval($profileExternalIds, $projectType, $type);

           $this->removeAllProjectApprovalInNotArrayProfileId($profileExternalIds, $projectType, $type);
        }else{
            $this->removeAllProjectApprovalInNotArrayProfileId(array(), $projectType, $type);
        }

        return $projectType;
    }

    public function validateControlUnique($projectTypeId, $control)
    {
        $validate = true;
        /** @var Entities\ProjectType $projectType */
        $projectType = $this->projectTypeRepository->findOneBy(array("externaltype" => $control));
        if( in_array($control,$this->uniqueControls) && $projectType && $projectType->getId() != $projectTypeId){
            $validate = false;
        }

        return $validate;
    }

    /**
     * @param array $profileExternalIds
     * @param $projectType
     * @param string $type
     */
    private function removeAllProjectApprovalInNotArrayProfileId(array $profileExternalIds,
                                                                 $projectType, $type = Entities\ProjectApproval::TYPE_AUTHORIZE)
    {
        /** @var Entities\ProjectApproval[] $projectApprovals */
        $projectApprovals = $this->projectApprovalRepository->findBy(array("projectType" => $projectType, "type" =>$type));
        foreach ($projectApprovals as $projectApproval) {
            if (!in_array($projectApproval->getProfile(), $profileExternalIds)) {
                $this->entityManager->remove($projectApproval);
                $this->entityManager->flush();
            }
        }
    }

    /**
     * @param $profileExternalIds
     * @param $projectType
     */
    private function createProjectApproval(array $profileExternalIds, $projectType, $type)
    {
        foreach ($profileExternalIds as $profileExternalId) {
            if (!empty($profileExternalId)) {
                $projectApproval = $this->projectApprovalRepository->findOneBy(array("projectType" => $projectType,
                    "profile" => $profileExternalId, "type" => $type));
                if (empty($projectApproval)) {
                    $projectApproval = $this->projectApprovalFactory->createProjectApproval($profileExternalId, $projectType, $type);
                    $this->entityManager->persist($projectApproval);
                }
            }
        }
        $this->entityManager->flush();
    }


    public function createProjectType(Entities\ProjectType $projectType = null, $allowedEndTask, $externalType, $label, $requiresApproval,
                                      $useMergeRequest, $useTimePlanning, $status = "A"){
        $projectType = $this->projectTypeFactory->createProjectType($projectType, $allowedEndTask, $externalType, $label, $requiresApproval,
            $useMergeRequest, $useTimePlanning, $status);
        $this->entityManager->persist($projectType);
        return $projectType;

    }

    public function RemoveProjectType($projectTypeId){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        $projectType = null;
        try {
            $projectType = $this->projectTypeRepository->find($projectTypeId);

            $this->validateRemoveProjectType($projectType);

            $this->removeProjectTypeInProjectDeleted($projectType);

            $this->removeAllForeignKeys($projectType);

            $this->entityManager->remove($projectType);
            $this->entityManager->flush();

            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
        return $projectType;

    }

    public function createProjectTypeFromController($projectTypeId, $allowedEndTask, $externalType, $label, $requiresApproval,
                                      $useMergeRequest, $useTimePlanning, $profileExternalIds, $profileExternalIdsUse, $status = "A")
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        $projectType = null;
        try {
            if ($projectTypeId) {
                /** @var Entities\ProjectType $projectType */
                $projectType = $this->projectTypeRepository->find($projectTypeId);
            }
            $projectType = $this->createProjectType($projectType, $allowedEndTask, $externalType, $label, $requiresApproval,
                $useMergeRequest, $useTimePlanning, $status);

            $this->entityManager->flush();

            $requiresApprovalValidation = $requiresApproval == Entities\ProjectApproval::REQUIRED;
            $this->setProfilesInProjectType($projectType, $profileExternalIdsUse, true, Entities\ProjectApproval::TYPE_USE);
            $this->setProfilesInProjectType($projectType, $profileExternalIds, $requiresApprovalValidation,
                Entities\ProjectApproval::TYPE_AUTHORIZE);

            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
        return $projectType;
    }

    /**
     * @param $projectType
     * @return null|object
     * @throws \Exception
     */
    protected function validateRemoveProjectType($projectType)
    {
        $project = $this->projectRepository->findOneBy(array('projectType' => $projectType));
        if ($project) {
            throw new \Exception(Messages::ERR_REMOVE_PROJECT_TYPE_USER_IN_PROJECT);
        }
        return $project;
    }

    /**
     * @param $projectType
     */
    protected function removeProjectTypeInProjectDeleted($projectType)
    {
        $this->entityManager->getFilters()->disable("logicalDelete");
        /** @var Entities\Project[] $projects */
        $projects = $this->projectRepository->findBy(array('projectType' => $projectType, 'status' => 'D'));
        $projectTypeDefault = Utilities::getProjectTypeDefault();
        foreach ($projects as $project) {
            $project->setProjectType($projectTypeDefault);
            $this->entityManager->persist($project);
        }
        $this->entityManager->flush();
        $this->entityManager->getFilters()->enable("logicalDelete");
    }

    /**
     * @param $projectType
     */
    protected function removeAllForeignKeys($projectType)
    {
        /** @var Entities\ClassificationProjectType[] $classificationProjectTypes */
        $classificationProjectTypes = $this->classificationProjectTypeRepository->findBy(array('projectType' => $projectType));
        foreach ($classificationProjectTypes as $classificationProjectType) {
            $this->entityManager->remove($classificationProjectType);
        }

        /** @var Entities\ProjectApproval[] $projectApprovals */
        $projectApprovals = $this->projectApprovalRepository->findBy(array('projectType' => $projectType));
        foreach ($projectApprovals as $projectApproval) {
            $this->entityManager->remove($projectApproval);
        }
        $this->entityManager->flush();
    }

    public function createTaskTypeProjectType($projectType, $taskType)
    {
        $taskTypeProjectTypeRepository = InstanceProvider::getRepository()->getTaskTypeProjectType();
        $taskTypeProjectTypeFactory = InstanceProvider::getFactory()->getTaskTypeProjectType();
        $taskTypeProjectType = $taskTypeProjectTypeRepository->findOneBy(
            array('projectType' => $projectType, 'taskType'=>$taskType));
        if(!$taskTypeProjectType){
            $taskTypeProjectType  = $taskTypeProjectTypeFactory->createTaskTypeProjectType($taskType,$projectType);
            $this->entityManager->persist($taskTypeProjectType);
        }
        return $taskTypeProjectType;
    }

} 