<?php
namespace Service;

use Zeedhi\Framework\Session\Session;
use Util\Utilities;

class Signup {

    protected $entityManager;
    protected $organizationService;
    protected $structureService;
    protected $profileService;
    protected $structureTypeService;
    protected $resourceService;
    protected $sessionManager;

    public function __construct(\Doctrine\ORM\EntityManager $entityManager, Organization $organizationService, Structure $structureService,
                                Profile $profileService, StructureType $structureTypeService, Resource $resourceService, Session $sessionManager){

        $this->entityManager = $entityManager;
        $this->organizationService = $organizationService;
        $this->structureService = $structureService;
        $this->profileService = $profileService;
        $this->structureTypeService = $structureTypeService;
        $this->resourceService = $resourceService;
        $this->sessionManager = $sessionManager;
    }

    protected function saveInitialOrganization($organizationData){
        $organizationFoundation = \DateTime::createFromFormat("d/m/Y", $organizationData['CREATION_ORG']);
        $organization = $this->organizationService->createOrganization( $organizationData['ORG'], $organizationFoundation, $organizationData['REGISTER'],
                                                                        $organizationData['ORG_DESC']);
        $this->sessionManager->set('organizationId', $organization->getId());
        return $organization;
    }

    protected function saveOrganizationStructure($organization, $administrator){
        $structureType = $this->structureTypeService->createStructureType(StructureType::ORGANIZATION_LABEL, StructureType::ORGANIZATION_CONTROL);
        $structure = $this->structureService->saveNewStructure($organization->getName(), $structureType, $administrator, null, Utilities::ACTIVE);
        return $structure;
    }

    protected function saveAdministrator($administratorData, $organization){
        $profile = $this->profileService->createProfile(Profile::EMPLOYEE_CATEGORY, Profile::ADMINISTRATOR_LABEL, Utilities::ACTIVE, null,
                                                        Profile::ADMINISTRATOR_CONTROL);
        $resource = $this->resourceService->createResource( Resource::WORK_RESOURCE, Resource::IS_USER_TRUE, Utilities::ACTIVE,
                                                            $administratorData['ADMIN_EMAIL'], $administratorData['ADMIN_PASSWORD'],
                                                            $administratorData['ADMIN_NAME'], null, Resource::CLASS_EMPLOYEE);
        $this->sessionManager->set('userId', $resource->getId());
        $this->resourceService->persistResourceProfile($resource, $profile);
        $structure = $this->saveOrganizationStructure($organization, $resource);
        $this->profileService->updateProfile(   $profile, Profile::EMPLOYEE_CATEGORY, $profile->getLabel(), $profile->getStatus(), $structure,
                                                $profile->getControl(), $resource->getId());
        $this->organizationService->updateOrganization( $organization, $organization->getName(), $organization->getFoundation(), $organization->getStatus(),
                                                        $organization->getDocument(), $organization->getDescription(), $resource->getId());
        $this->resourceService->updateResource( $resource, $resource->getResourcesType(), $resource->getIsUser(), $resource->getStatus(),
                                                $resource->getEmail(), $resource->getName(), $resource->getDescription(), $resource->getClass(),
                                                $resource->getNickName(), $resource->getDocumentNumber(), $resource->getPhone(), $resource->getAddress(),
                                                $resource->getDistrict(), $resource->getCity(), $resource->getZipCode(), $resource->getParent(),
                                                $resource->getId());
    }

    protected function saveAdditionalProfiles(){
        $this->profileService->createProfile(   Profile::EMPLOYEE_CATEGORY, Profile::PROJECT_MANAGER_LABEL, Utilities::ACTIVE, null,
                                                Profile::PROJECT_MANAGER_CONTROL);
        $this->profileService->createProfile(Profile::EMPLOYEE_CATEGORY, Profile::RESOURCE_LABEL, Utilities::ACTIVE, null, Profile::RESOURCE_CONTROL);
        $this->profileService->createProfile(Profile::SPONSOR_CATEGORY, Profile::CUSTOMER_LABEL, Utilities::ACTIVE, null, Profile::CUSTOMER_CONTROL);
    }

    protected function saveAdditionalStructureTypes(){
        $this->structureTypeService->createStructureType(StructureType::BUSINESS_LABEL, StructureType::BUSINESS_CONTROL);
        $this->structureTypeService->createStructureType(StructureType::TEAM_LABEL, StructureType::TEAM_CONTROL);
    }

    protected function saveDefaults(){
        $this->saveAdditionalProfiles();
        $this->saveAdditionalStructureTypes();
    }

    public function save($organizationData, $administratorData){
        $connection = $this->entityManager->getConnection();
        try{
            $connection->beginTransaction();
            $organization = $this->saveInitialOrganization($organizationData);
            $this->saveAdministrator($administratorData, $organization);
            $this->saveDefaults();
            $connection->commit();
        }
        catch(\Exception $e){
            throw $e;
        }
    }
} 