<?php
/**
 * Created by PhpStorm.
 * User: tuliomagalhaes
 * Date: 28/07/14
 * Time: 13:47
 */

namespace Service;


use Doctrine\ORM\EntityManager;
use Util\Utilities;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Zeedhi\DevOps\Model\Projectmanagement\Repositories;

class TaskPrecursor {

    protected  $entityManager;
    protected  $environment;
    protected  $taskPrecursorFactory;

    const BEGIN_END_PRECURSOR = 'IT';
    const END_BEGIN_PRECURSOR = 'TI';
    const END_END_PRECURSOR = 'TT';
    const BEGIN_BEGIN_PRECURSOR = 'II';

    const PRECURSOR_TYPE = Entities\TaskRelationship::PRECURSOR_TYPE;
    const CONFLICT_TYPE = Entities\TaskRelationship::CONFLICT_TYPE;
    const CONFLICT_DEVOPS_TYPE = Entities\TaskRelationship::CONFLICT_DEVOPS_TYPE;
    const ERROR_TYPE = Entities\TaskRelationship::ERROR_TYPE;
    const ERROR_MR_TYPE = Entities\TaskRelationship::ERROR_MR_TYPE;
    const TEST_TYPE = Entities\TaskRelationship::TEST_TYPE;
    const ORIGIN_TYPE = Entities\TaskRelationship::ORIGIN_TYPE;
    const RELATIONSHIP_ISSUE_TYPE = Entities\TaskRelationship::RELATIONSHIP_ISSUE_TYPE;
    const MERGE_REQUEST_TYPE = Entities\TaskRelationship::MERGE_REQUEST_TYPE;
    const REPLICATION_TYPE = Entities\TaskRelationship::REPLICATION_TYPE;
    const ISSUE_WITH_ISSUE_TYPE = Entities\TaskRelationship::ISSUE_WITH_ISSUE_TYPE;

    const lengthMaxDescription  = 2500;
    const STRING_GENERATED_DEVOPS = "-- Generated by DevOps! --";
    const STRING_LIMIT_DESCRIPTION = "\n\n-- Descrição das tarefas geradoras superior ao limite. Verifique as mesmas caso necessário. --";
    /** @var  Repositories\Task */
    protected $taskRepository;
    /** @var Repositories\TaskRelationship  */
    protected $taskRelationshipRepository;
    /** @var  array */
    protected $arrayGeneratedTypeRelationship;

    public static function getGeneratedTypRelationship() {
        return array(
            self::MERGE_REQUEST_TYPE,
            self::ERROR_TYPE,
            self::REPLICATION_TYPE,
            self::CONFLICT_TYPE,
            self::CONFLICT_DEVOPS_TYPE,
            self::ERROR_MR_TYPE,
            self::TEST_TYPE
        );
    }

    public static function getGeneratedTypRelationshipWithoutBranch() {
        return array(
            self::ERROR_TYPE,
            self::CONFLICT_DEVOPS_TYPE,
            self::ERROR_MR_TYPE,
        );
    }

    public function __construct(EntityManager $entityManager, \Util\Utilities $environment, \Factory\TaskPrecursor $taskPrecursorFactory){
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->taskPrecursorFactory = $taskPrecursorFactory;
        $this->taskRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'Task');
        $this->taskRelationshipRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'TaskRelationship');
        $this->taskBranchesRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'TaskBranches');
        $this->arrayGeneratedTypeRelationship = self::getGeneratedTypRelationship();
        $this->arrayTypeRelationshipGetDescription = array(self::ERROR_TYPE,self::ERROR_MR_TYPE);
    }

    /**
     * Creates task relationship.
     *
     * @param $task
     * @param $taskPrecursor
     * @param $type
     * @param null $typePrecursor
     * @param bool $isFlushNeeded
     * @return Entities\TaskRelationship
     */
    public function createTaskPrecursor($task,$taskPrecursor,$type,$typePrecursor = null, $isFlushNeeded = true) {
        $entity = $this->taskPrecursorFactory->createTaskPrecursor($task,$taskPrecursor,$type,$typePrecursor);
        $this->entityManager->persist($entity);
        if($isFlushNeeded){
            $this->entityManager->flush();
        }
        return $entity;
    }

    /**
     * Updates task relationship.
     *
     * @param $entity
     * @param $task
     * @param $taskPrecursor
     * @param $type
     * @param null $typePrecursor
     * @return Entities\TaskRelationship
     */
    public function updateTaskPrecursor($entity,$task,$taskPrecursor,$type,$typePrecursor = null) {

        $entity = $this->taskPrecursorFactory->updateTaskPrecursor($entity,$task,$taskPrecursor,$type,$typePrecursor);

        $this->entityManager->persist($entity);
        $this->entityManager->flush();

        return $entity;

    }

    public function setDescriptionByTaskRelationship($taskId, $type, $originTaskId)
    {
        /** @var Entities\Task $task */
        $task = $this->taskRepository->find($taskId);
        if ($task) {
            $description = $task->getDescription();
            $initDescription = $task->getDescription();
            $descDOHasBreakLine = strpos($description, "\n\n" . self::STRING_GENERATED_DEVOPS);
            $stringDevOps = $descDOHasBreakLine ? "\n\n" . self::STRING_GENERATED_DEVOPS : self::STRING_GENERATED_DEVOPS;
            $description = str_replace($stringDevOps, '', $description, $count);
            $description .= $type == Entities\TaskRelationship::MERGE_REQUEST_TYPE ? "\n\n" : '';
            $hasOriginDescription = strpos($description, '(Origem)');
            $description .= $this->getAllDescriptionTask($task, '', !empty($description), $description, $type, $originTaskId, $taskId, $hasOriginDescription);
            if ($count >= 1) {
                $description .= "\n\n" . self::STRING_GENERATED_DEVOPS;
            }
            if((strlen($description) > self::lengthMaxDescription)) {
                $description = strlen($initDescription . self::STRING_LIMIT_DESCRIPTION) <= self::lengthMaxDescription
                    ? $initDescription . self::STRING_LIMIT_DESCRIPTION : $initDescription;
            }
            $task->setDescription($description);
            $this->entityManager->persist($task);
            $this->entityManager->flush();
        }
    }

    public function getAllDescriptionTask(Entities\Task $task, $description, $hasDescription, $originDescription, $type,
                                          $originTaskId, $taskId, $hasOriginDescription)
    {
        $descriptionTemp = '';
        /** @var Entities\TaskRelationship[] $taskRelationships */
        $taskRelationships = $this->taskRelationshipRepository->findBy(array('taskRelationship' => $task,
            'type' => Entities\TaskRelationship::getAllTypesForMergeDescription()));

        foreach ($taskRelationships as $taskRelationship) {
            if($type == Entities\TaskRelationship::TEST_TYPE && $taskRelationship->getType() == Entities\TaskRelationship::ERROR_MR_TYPE) {
                return self::getMergeRequestDescriptionForTest($taskRelationship->getTask());
            }
            $descriptionTemp = $description;
            if (in_array($taskRelationship->getType(), $this->arrayTypeRelationshipGetDescription) ||
                ($type == Entities\TaskRelationship::MERGE_REQUEST_TYPE &&
                 $taskId != $taskRelationship->getTaskRelationship()->getId() &&
                 $taskRelationship->getType() != Entities\TaskRelationship::MERGE_REQUEST_TYPE)) {
                $addOriginMergeRequest = $originTaskId == $taskRelationship->getTaskRelationship()->getId() && !$hasOriginDescription;
                $id = $taskRelationship->getTaskRelationship()->getId();
                if(strpos($description, "Descrição da tarefa $id") === false) {
                    if($addOriginMergeRequest) {
                        $description = str_replace('(Origem)', '', $description);
                    }
                    $descriptionTemp = self::addDescription($taskRelationship->getTaskRelationship(), true, $addOriginMergeRequest) . $descriptionTemp;
                }
            }

            if ((!(strlen($descriptionTemp . $originDescription) > 2500) &&
                   $taskRelationship->getType() != Entities\TaskRelationship::MERGE_REQUEST_TYPE &&
                   $taskRelationship->getType() != Entities\TaskRelationship::ERROR_MR_TYPE &&
                   $taskRelationship->getType() != Entities\TaskRelationship::ORIGIN_TYPE) ||
                (!(strlen($descriptionTemp . $originDescription) > 2500) &&
                   $taskRelationship->getType() == Entities\TaskRelationship::MERGE_REQUEST_TYPE &&
                   $originTaskId != $taskRelationship->getTaskRelationship()->getId())) {
                $description = $this->getAllDescriptionTask($taskRelationship->getTask(), $descriptionTemp, $hasDescription, $originDescription, $type, $originTaskId, $taskId, $hasOriginDescription);
            }
        }

        $id = $task->getId();
        if (empty($taskRelationships) &&
            strlen(self::addDescription($task, $hasDescription) . $description . $originDescription) <= 2500
        ) {
            $addOriginMergeRequest = $originTaskId == $id;
            if(strpos($description, "Descrição da tarefa $id") === false) {
                if($addOriginMergeRequest) {
                    $description = str_replace('(Origem)', '', $description);
                }
                $description = self::addDescription($task, $hasDescription, $addOriginMergeRequest) . $description;
            }
        } else if($type == Entities\TaskRelationship::MERGE_REQUEST_TYPE &&
                  $id == $originTaskId) {
            if(strpos($description, "Descrição da tarefa $id") === false) {
                $description .= $descriptionTemp;
            }
        }

        return trim($description);
    }

    static protected function addDescription(Entities\Task $task, $hasDescription = true, $addOriginMergeRequest = false){
        $description = $task->getDescription();
        $id = $task->getId();
        $space = $hasDescription ? "\n\n" : '';
        $originDescription = $addOriginMergeRequest ? '(Origem)' : '';

        $description = "$space-- Descrição da tarefa $id $originDescription --\n$description";
        return $description;
    }

    static protected function getMergeRequestDescriptionForTest(Entities\Task $task) {
        $description = $task->getDescription();
        $pos = strpos($description, '-- Descrição da tarefa');
        return $pos ? substr($description, $pos - 1) : $description;
    }

    /**
     * @param $taskId
     * @return string
     * @throws \Doctrine\DBAL\ConnectionException
     * @throws \Exception
     */
    public function updateRelationshipMergeRequest($taskId) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();

        try {
            /** @var Entities\TaskRelationship $errorRelationship */
            $errorRelationship = $this->taskRelationshipRepository->findOneBy(array('taskRelationship' => $taskId,
                                                                                    'type' => self::ERROR_MR_TYPE));

            $taskIds = $this->getUpdateRelationshipMRTaskIds($errorRelationship);
            $branchName = $this->getBranchNameToReturn($taskIds[0]);
            /** @var Entities\TaskRelationship $mergeRelationship[] */
            $mergeRelationships = $this->taskRelationshipRepository->findBy(array('taskRelationship' => $taskIds,
                                                                                  'type' => self::MERGE_REQUEST_TYPE));
            foreach($mergeRelationships as $mergeRelationship) {
                $this->setOriginRelationship($mergeRelationship);
                $this->createNewMergeRequestRelationship($mergeRelationship);
            }

            $connection->commit();
            return $branchName;
        } catch(\Exception $e){
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * @param $mainTaskId
     * @return string
     */
    private function getBranchNameToReturn($mainTaskId)
    {
        /** @var Entities\TaskRelationship $relationship */
        $relationship = $this->taskRelationshipRepository->findOneBy(array('taskRelationship' => $mainTaskId,
                                                                           'type' => self::MERGE_REQUEST_TYPE));

        $relatedTasks = $this->taskRelationshipRepository->findRelatedTaskIds($relationship->getTask()->getId());
        if(!empty($relatedTasks)) {
            $taskId = $relatedTasks['MAIN_TASK'];
        } else {
            $taskId = $relationship->getTask()->getId();
        }
        /** @var Entities\Task $task */
        $task = $this->taskRepository->find($taskId);
        $branchInfo = $task->getTaskBranchRecursive();
        return $branchInfo['BRANCH_NAME'];
    }
    public function saveFirstRelationshipById($taskId,$taskRelationshipId,$typeRelationship){
        $task = $this->taskRepository->find($taskId);
        $taskRelationship = $this->taskRepository->find($taskRelationshipId);
        $this->saveFirstRelationship($task,$taskRelationship,$typeRelationship);
    }

    public static function firstRelationshipOrderTask(Entities\Task $task,Entities\Task $taskRelationship){
        $return = array();
        if($task->getId() > $taskRelationship->getId()){
            $return[] = $task;
            $return[] = $taskRelationship;
        }else{
            $return[] = $taskRelationship;
            $return[] = $task;
        }
        return $return;
    }

    public static function invertFirstRelationshipOrderTask(Entities\Task $task,Entities\Task $taskRelationship){
        return array($taskRelationship,$task);
    }

    public function saveFirstRelationship(Entities\Task $task,Entities\Task $taskRelationship,$typeRelationship){
        if(in_array($typeRelationship,self::getGeneratedTypRelationship())){
            if($typeRelationship == Entities\TaskRelationship::MERGE_REQUEST_TYPE){
                list($task,$taskRelationship) = self::invertFirstRelationshipOrderTask($task,$taskRelationship);
            }else{
                list($task,$taskRelationship) = self::firstRelationshipOrderTask($task,$taskRelationship);
            }
            $taskRelationship = $this->setTaskRelationshipInFirstRelationship($task, $taskRelationship);

            $firstTaskRelationship = $this->taskRelationshipRepository->findOneBy(array( 'taskRelationship' => $taskRelationship,
                'task' => $task,'type' => Entities\TaskRelationship::FIRST_TYPE));
            if(!$firstTaskRelationship){
                $firstTaskRelationship = $this->taskPrecursorFactory->createTaskPrecursor($task, $taskRelationship,
                    Entities\TaskRelationship::FIRST_TYPE);
                $this->entityManager->persist($firstTaskRelationship);
                $this->entityManager->flush();
            }
        }
    }

    /**
     * @param Entities\TaskRelationship $relationship
     */
    private function createNewMergeRequestRelationship(Entities\TaskRelationship $relationship)
    {
        /** @var Entities\TaskRelationship $errorMrRelationship */
        $errorMrRelationships = $this->taskRelationshipRepository->findBy(array('task' => $relationship->getTaskRelationship(),
                                                                                'type' => self::ERROR_MR_TYPE));
        $errorMrRelationship = null;
        /** @var Entities\TaskRelationship $errMrRelationships */
        foreach ($errorMrRelationships as $errMrRelationships) {
            if (!$errorMrRelationship ||
                $errorMrRelationship->getTaskRelationship()->getId() < $errMrRelationships->getTaskRelationship()->getId()
            ) {
                $errorMrRelationship = $errMrRelationships;
            }
        }

        $newErrorRelationship = $this->taskPrecursorFactory->createTaskPrecursor($errorMrRelationship->getTaskRelationship(),
            $errorMrRelationship->getTask(), self::MERGE_REQUEST_TYPE, null);
        $this->entityManager->persist($newErrorRelationship);
        $this->entityManager->flush();

        $this->setDescriptionByTaskRelationship($errorMrRelationship->getTask()->getId(),
            self::MERGE_REQUEST_TYPE, $errorMrRelationship->getTaskRelationship()->getId());
    }

    /**
     * @param Entities\TaskRelationship $relationship
     */
    private function setOriginRelationship(Entities\TaskRelationship $relationship)
    {
        $relationship->setType(self::ORIGIN_TYPE);
        $this->entityManager->persist($relationship);
    }

    /**
     * @param Entities\TaskRelationship $relationship
     * @return array
     */
    private function getUpdateRelationshipMRTaskIds(Entities\TaskRelationship $relationship)
    {
        $taskId = $relationship->getTask()->getId();
        $relatedTasks = $this->taskRelationshipRepository->findRelatedTaskIds($taskId);
        if (!empty($relatedTasks)) {
            $taskIds = explode(',', $relatedTasks['MAIN_TASK'] . ',' . $relatedTasks['RELATED_TASKS']);
            return $taskIds;
        } else {
            $taskIds = array($relationship->getTask()->getId());
            return $taskIds;
        }
    }

    /**
     * @param $task
     * @param $taskRelationship
     * @return array
     */
    public function setTaskRelationshipInFirstRelationship($task, $taskRelationship)
    {
        /** @var Entities\TaskRelationship $firstTaskRelationship */
        $firstTaskRelationship = $this->taskRelationshipRepository->findOneBy(array('task' => array($task, $taskRelationship),
            'type' => Entities\TaskRelationship::FIRST_TYPE));
        if ($firstTaskRelationship) {
            $taskRelationship = $firstTaskRelationship->getTaskRelationship();
        }
        return $taskRelationship;
    }
    
    public function taskRelationshipWithGeneratingTaskIssue($originTaskId, $taskRelationshipId)
    {
        $issuesRelationship = $this->taskRelationshipRepository->findBy(
            array('taskRelationship' => $originTaskId, 'type' => 'I')
        );
        
        $taskCreated = $this->taskRepository->find($taskRelationshipId);
        
        foreach($issuesRelationship as $issueRelationship){
            $issue = $this->taskRepository->find($issueRelationship->getTask());
            $this->createTaskPrecursor(
                $issue, $taskCreated, TaskPrecursor::RELATIONSHIP_ISSUE_TYPE, null, true
            );
        }
        
    }
} 