/*****************************************************************************
* Sistemas Integrados TECNISA                                                *
*                                                                            *
* Compras e Suprimentos                                                      *
*                                                                            *
******************************************************************************
*                                                                            *
* Nome: SEMANAL_PROX_DATA_ENTREGA                                            *
*                                                                            *
* Descrio:                                                                 *
* Calcula datas de entrega para um determinado mes baseando-se na            *
* Periodicidade de Entrega e nos Feriados de Compras.                        *
*                                                                            *
*                                                                            *
* Criao:                                                                   *
* 03/01/2000 - Yoris L. de Souza                                             *
*                                                                            *
* Manutenes:                                                               *
*   /  /   -                                                                 *
*                                                                            *
*                                                                            *
******************************************************************************

******************************************************************************
* PARAMETROS:                                                                *
*                                                                            *
* P_CDFILIAL     - Cdigo da Filial                                          *
* P_CDPERIENTR   - Cdigo da Periodicidade de Entrega                        *
* P_DTINICIAL    - Data do Inicio do Mes                                     *
* P_IDDIANUTIL   - Considera dia nao util                                    *
******************************************************************************/

IF EXISTS(SELECT * FROM sysobjects O, sysusers U
          WHERE 
            o.name = 'SEMANAL_PROX_DATA_ENTREGA'  and 
            type = 'P'                  and
            u.uid = o.uid               and 
            u.name = USER)
  DROP PROCEDURE SEMANAL_PROX_DATA_ENTREGA
  GO


CREATE PROCEDURE SEMANAL_PROX_DATA_ENTREGA
  @P_CDFILIAL     VARCHAR(4),
  @P_CDPERIENTR   VARCHAR(2),
  @P_DTINICIAL    DATETIME,
  @P_IDDIANUTIL   VARCHAR(1),
  @P_DTENTRAUX    DATETIME OUTPUT
AS
   DECLARE @V_CONTAUXI   NUMERIC(10);
   DECLARE @V_DIENTREGA  INT;
   DECLARE @V_DTENTRAUX  DATETIME;
   DECLARE @V_DTAUX      DATETIME;
   DECLARE @V_DTVERAUX   DATETIME;
   DECLARE @V_IDDIASEM   INT;

/*--- CURSOR QUE CONTEM AS DATAS DE ENTREGA DO PROXIMO MES    ------*/
/*-   COM BASE NOS DIAS DE ENTREGA SEMANAIS                   ------*/
 DECLARE C_DATAS_ENTR SCROLL CURSOR FOR
   (SELECT
      CONVERT(INT,DIENTREGA),
      1 AS OCORR
    FROM DIASENTR
    WHERE
      CDPERIENTR = @P_CDPERIENTR AND
      IDOCOR01 = '1'
   UNION
   SELECT
     CONVERT(INT,DIENTREGA),
     2  AS OCORR
   FROM DIASENTR
     WHERE
      CDPERIENTR = @P_CDPERIENTR AND
      IDOCOR02 = '1'
   UNION
   SELECT
     CONVERT(INT,DIENTREGA),
     3  AS OCORR
   FROM DIASENTR
   WHERE
     CDPERIENTR = @P_CDPERIENTR AND
     IDOCOR03 = '1'
   UNION
   SELECT
     CONVERT(INT,DIENTREGA),
     4 AS OCORR
   FROM DIASENTR
   WHERE
     CDPERIENTR = @P_CDPERIENTR AND
     IDOCOR04 = '1'
   UNION
   SELECT
     CONVERT(INT,DIENTREGA),
     5 AS OCORR
   FROM DIASENTR
   WHERE
     CDPERIENTR = @P_CDPERIENTR AND
     IDOCOR05 = '1')
   ORDER BY 2;


BEGIN
    SET @V_DTENTRAUX = @P_DTINICIAL;

    OPEN C_DATAS_ENTR;
    FETCH NEXT FROM C_DATAS_ENTR INTO @V_DIENTREGA, @V_IDDIASEM;
    CLOSE C_DATAS_ENTR;
    DEALLOCATE C_DATAS_ENTR;

    EXEC ULTIMO_DIA_MES @P_DTDATA = @V_DTENTRAUX OUTPUT;
    SET @V_DTAUX = @V_DTENTRAUX;
    IF @V_IDDIASEM = 5
    BEGIN
       SET @V_DTENTRAUX = @V_DTENTRAUX + 1;
       EXEC ULTIMO_DIA_MES @P_DTDATA = @V_DTENTRAUX OUTPUT;
       SET @V_DTENTRAUX = @V_DTENTRAUX - 7;
       EXEC PROXIMO_DIA @V_DIENTREGA,1,@P_DTPROXIMA = @V_DTENTRAUX OUTPUT;
    END
    ELSE
    BEGIN
       EXEC PROXIMO_DIA @V_DIENTREGA,@V_IDDIASEM,@P_DTPROXIMA = @V_DTENTRAUX OUTPUT;
    END;

    SET @V_DTVERAUX = @V_DTENTRAUX;

    IF @V_IDDIASEM = 5
    BEGIN
      SET @V_DTVERAUX = @V_DTAUX + 1;
      EXEC ULTIMO_DIA_MES @P_DTDATA = @V_DTVERAUX OUTPUT;

      EXEC PROXIMO_DIA @V_DIENTREGA,1,@P_DTPROXIMA = @V_DTVERAUX OUTPUT ;
    END;

    IF (((MONTH(@V_DTVERAUX) = MONTH(@V_DTAUX + 1)) AND (@V_IDDIASEM > 1)) OR
        (@V_IDDIASEM = 1))
    BEGIN

       IF @P_IDDIANUTIL = 'S'
       BEGIN
         SET @V_CONTAUXI = 1;
         WHILE (@V_CONTAUXI > 0)
         BEGIN
           SET @V_CONTAUXI = 0;
           SELECT @V_CONTAUXI = COUNT(*)
           FROM FERICOMP
           WHERE
           CDFILIAL   = @P_CDFILIAL AND
           DTFERICOMP = @V_DTENTRAUX;
           IF (@V_CONTAUXI > 0)
           BEGIN
             SET @V_DTENTRAUX = @V_DTENTRAUX- 1;
           END;
         END;
       END;
    END;
    SET @P_DTENTRAUX = @V_DTENTRAUX - 1;
END;
GO
