/*+
|                                               |
|          Procedure: ETL_FATUINVOICE           |
|                                               |
| Rotina para insero de dados de BI -         |
| valores de Faturamento / Comissaria,          |
| para visualizao no CVI.                     |
|                                               |
|            Tabelas Preenchidas:               |
|                                               |
|       1:   ETLFATUINVOICE - Faturamento       |
|                                               |
|     Criao:                                  |
|     24/10/2007 - Breno de Assis e Castro      |
|                                               |
+*/


IF EXISTS (SELECT * 
	   FROM   SYSOBJECTS O, SYSUSERS U
           WHERE     
	          O.NAME = 'ETL_FATUINVOICE' 
           AND    TYPE   = 'P' 	              
	   AND    U.UID  = O.UID               
	   AND    U.NAME = USER
          )
	  DROP PROCEDURE ETL_FATUINVOICE
GO  
CREATE PROCEDURE ETL_FATUINVOICE
                          @P_DTINI       DATETIME, 
                          @P_DTFINA      DATETIME 
AS 

   DECLARE @V_DTFIM                  DATETIME; 
   DECLARE @V_DTINI                  DATETIME; 
                                                                                         
---------------VALOR CURSOR C_ETL--------------------
   DECLARE @V_CDFILIAL               VARCHAR(04);
   DECLARE @V_CDAREAGERE             VARCHAR(10);
   DECLARE @V_CDSEGMER               VARCHAR(10);
   DECLARE @V_CDEMPRESA              VARCHAR(02);
   DECLARE @V_CDSUPERVISOR           VARCHAR(12);
   DECLARE @V_DTINICIALBI            VARCHAR(25);
   DECLARE @V_CDGOPERAC              VARCHAR(12);
   DECLARE @V_CDPAIS                 VARCHAR(04);
   DECLARE @V_CDREGIAO               VARCHAR(04);
                                                                         
---------------VALOR CURSOR C_CLIENTE----------------
   DECLARE @V_CDCLIENTE              VARCHAR(21);                                                                 
                                     
---------------CURSOR C_VOICE------------------------
   DECLARE @V_CDFILIAL_VOICE         VARCHAR(04);
   DECLARE @V_CDCLIENTE_VOICE        VARCHAR(21);
   DECLARE @V_VRFATURAMENTO          DOUBLE PRECISION;
   DECLARE @V_VRMERCADORIALI         DOUBLE PRECISION;
   DECLARE @V_VRTXAERO               DOUBLE PRECISION;
   DECLARE @V_VRHANDLINGLI           DOUBLE PRECISION;
   DECLARE @V_VRITEMENCA             DOUBLE PRECISION;
   DECLARE @V_VRITEMENCAAC           DOUBLE PRECISION;
   DECLARE @V_VRIMPOSTOS             DOUBLE PRECISION;
                                                        
-----------------------------------------------------                                                                                                                        
   DECLARE @V_MES                    VARCHAR(02); 
   DECLARE @V_ANO                    VARCHAR(04);    
   
   DECLARE @V_ZERO                   NUMERIC;
   DECLARE @V_CONT                   NUMERIC;
-----------------------------------------------------                                                              
BEGIN
   

   SET @V_MES  =  (CONVERT (VARCHAR, MONTH (@P_DTINI)));                                            
   SET @V_ANO  =  (CONVERT (VARCHAR,  YEAR (@P_DTINI)));                                          
                                                 
   BEGIN TRANSACTION
   DELETE FROM ETLFATUINVOICE                                                  
   WHERE  (ANETLFAC > @V_ANO)
   OR     (ANETLFAC = @V_ANO AND MEETLFAC >= @V_MES);                                                 
   COMMIT TRANSACTION;                                                                   
                  
   SET @V_DTINI = CONVERT(DATETIME,('01/'+@V_MES+'/'+@V_ANO),103);

   /*INICIO WHILE LOOP VERIFICA DATA INICIAL / DATA FINAL*/
   WHILE (@V_DTINI <= @P_DTFINA) BEGIN
      BEGIN TRANSACTION                                            
                                         
         SET @V_DTFIM = DATEADD(MM,1,@V_DTINI) - DAY(@V_DTINI);                                            
                                                                                 
               /*INICIO WHILE CURSOR C_ETL */                  
         DECLARE C_ETL CURSOR FOR
           (SELECT 
                   F.CDFILIAL,        F.CDAREAGERE,        F.CDSEGMER,
                   F.CDEMPRESA,       F.CDSUPERVISOR,      F.DTINICIALBI,
                   F.CDGOPERAC,       EMP.CDPAIS 
            FROM   
                   FILIAL F , EMPRESA EMP
            WHERE  
                   F.CDEMPRESA = EMP.CDEMPRESA
            AND    F.IDFUNCAOFILI IS NULL     
            AND    F.DTINICIALBI IS NOT NULL 
            ); 
            OPEN C_ETL;
            FETCH NEXT FROM C_ETL INTO
                                     @V_CDFILIAL,     @V_CDAREAGERE,      @V_CDSEGMER,
                                     @V_CDEMPRESA,    @V_CDSUPERVISOR,    @V_DTINICIALBI,
	                             @V_CDGOPERAC,    @V_CDPAIS;
            
            WHILE(@@FETCH_STATUS = 0) BEGIN 
                                                                 
               IF CONVERT (DATETIME, ('01' + '/' + CONVERT(VARCHAR, MONTH(@V_DTINICIALBI))+'/'+ CONVERT (VARCHAR, YEAR(@V_DTINICIALBI)))) <= @V_DTINI BEGIN     
                  DECLARE C_CLIENTE CURSOR FOR
                    (SELECT                
	                    NF.CDCLIENTE
	             FROM       
	                    NOTAFISCAL NF, COMPANHIA CO, CIAFILI CF
	             WHERE     
	                    NF.CDEMPRESA  = @V_CDEMPRESA      
	             AND    CF.CDCIA      = CO.CDCIA      
	             AND    NF.CDCLIENTE  = CO.CDCLIENTE
	             AND    NF.CDFILIAL   = @V_CDFILIAL
	             AND    NF.CDCLIENTE IS NOT NULL 
	             AND    NF.IDSTATUS  IN ('A','E')   
	             AND    ISNULL(@V_CDFILIAL, CF.CDFILIAL) = CF.CDFILIAL 
	             AND    NF.DTEMISSAO BETWEEN @V_DTINI AND @V_DTFIM 
	             GROUP BY 
	                    NF.CDCLIENTE
                    );      
                  OPEN C_CLIENTE;
                  FETCH NEXT FROM C_CLIENTE INTO 
                                                 @V_CDCLIENTE;
                  WHILE (@@FETCH_STATUS = 0) BEGIN         
                  
                  DECLARE C_VOICE CURSOR FOR
                    (SELECT 
                            NF.CDFILIAL,
                            CL.CDCLIENTE,
                           (SUM ( ISNULL( RE.VRMERCADORIALI ,0 ))+   SUM(ISNULL(HD.VRHANDLINGLI,0))+ SUM(ISNULL(TA.VRTXAERO,0))+ SUM(ISNULL(AC.VRITEMENCAAC,0))- SUM(ISNULL(IE.VRITEMENCA,0))) AS VRFATURAMENTO,
                            SUM ( ISNULL( RE.VRMERCADORIALI ,0 ))    AS VRMERCADORIALI, 
                            SUM ( ISNULL( HD.VRHANDLINGLI   ,0 ))    AS VRHANDLINGLI, 
                            SUM ( ISNULL( TA.VRTXAERO       ,0 ))    AS VRTXAERO,              
                            SUM ( ISNULL( IE.VRITEMENCA     ,0 ))    AS VRITEMENCA , 
                            SUM ( ISNULL( AC.VRITEMENCAAC   ,0 ))    AS VRITEMENCAAC,  
                            SUM ( ISNULL( IM.VRIMPOSTOS     ,0 ))    AS VRIMPOSTOS
                     FROM   
                            CLIENTE AS CL,                      
                           (SELECT       
                                   NF.CDCLIENTE,    NF.NRLANCTONF,   CLI.NMRAZSOCCLIE,  
                                   NF.DTENTRSAID,   NF.CDFILIAL
                            FROM    
                                   CLIENTE CLI, NOTAFISCAL NF
                            WHERE    
                                   CLI.CDCLIENTE = NF.CDCLIENTE
                            AND    NF.CDFILIAL   = @V_CDFILIAL 
                            AND    NF.CDCLIENTE  = @V_CDCLIENTE
                            AND    NF.CDCLIENTE IS NOT NULL          
                            AND    NF.DTENTRSAID BETWEEN @V_DTINI AND @V_DTFIM
                           ) AS NF,     
                                                                                   
--------------------------------------------------------------------------------                         
                      ----------VALOR  MERCADORIA----------
                           (SELECT DISTINCT 
                                   IP.CDFILIAL,       CLI.CDCLIENTE,     IP.NRSEQITPROD,         
			           CLI.NMRAZSOCCLIE,  NF.NRLANCTONF,     NF.DTENTRSAID, 
			           SUM(ISNULL( IP.VRBRUTITEM ,0))   AS   VRMERCADORIA,
			           SUM(ISNULL( IP.VRTOTAITEM ,0))   AS   VRMERCADORIALI
			    FROM    
			           ITEMPRODUTO IP,   PARAMCOMIS PG, 
			           NOTAFISCAL NF,    CLIENTE CLI
			    WHERE  
			         ( IP.CDFILIAL    =  @V_CDFILIAL        )
			    AND  ( NF.CDCLIENTE   =  @V_CDCLIENTE       ) 
			    AND  ( NF.CDCLIENTE   =  CLI.CDCLIENTE      )  
			    AND  ( IP.CDFILIAL    =  NF.CDFILIAL        ) 
			    AND  ( NF.CDFILIAL    =  PG.CDFILIAL        ) 
			    AND  ( IP.NRLANCTONF  =  NF.NRLANCTONF      ) 
			    AND  ( IP.CDFILIAL    =  PG.CDFILIAL        )
			    AND  ( NF.DTENTRSAID BETWEEN @V_DTINI AND @V_DTFIM ) 
			    AND  ((PG.CDPRODUTO  IS NULL) OR (IP.CDPRODUTO <> PG.CDPRODUTO ))  
			    AND  ((PG.CDPRAEROIN IS NULL) OR (IP.CDPRODUTO <> PG.CDPRAEROIN)) 
			    AND  ( NOT EXISTS (SELECT 
			                              NRSEQITPROD
			                       FROM   
			                              ITEMPROIMPOS II
			                       WHERE 
			                              II.CDFILIAL    = IP.CDFILIAL    
			                       AND    II.NRLANCTONF  = IP.NRLANCTONF  
			                       AND    II.NRSEQITPROD = IP.NRSEQITPROD 
			                       AND    II.CDIMPOSTO   = '31'
			                      )
			         )
			    GROUP BY                         
			            IP.CDFILIAL, 
             		            CLI.CDCLIENTE,	     
		             	    IP.NRSEQITPROD,
			            IP.NRLANCTONF,     
			            CLI.NMRAZSOCCLIE,  
			            NF.NRLANCTONF,     
			            NF.DTENTRSAID  
                            ) AS RE,
                              
--------------------------------------------------------------------------------     
                     --------VALOR SERVIOS--------   
                           (SELECT DISTINCT 
                                   IP.NRLANCTONF,       NF.CDCLIENTE, 
                                   CLI.NMRAZSOCCLIE,    NF.DTENTRSAID,
                                   SUM(IP.VRBRUTITEM) AS VRHANDLING,
                                   SUM(IP.VRTOTAITEM) AS VRHANDLINGLI
                            FROM   
                                   ITEMPRODUTO IP,   PARAMCOMIS PG, 
                                   ITEMPROIMPOS II,  NOTAFISCAL NF, 
                                   CLIENTE CLI
                            WHERE   
                                 ( IP.CDFILIAL     =  @V_CDFILIAL    )
                            AND  ( NF.CDCLIENTE    =  @V_CDCLIENTE   )      
                            AND  ( NF.CDCLIENTE    =  CLI.CDCLIENTE  )      
                            AND  ( IP.CDFILIAL     =  NF.CDFILIAL    ) 
                            AND  ( NF.CDFILIAL     =  PG.CDFILIAL    )  
                            AND  ( PG.CDFILIAL     =  II.CDFILIAL    ) 
                            AND  ( II.NRLANCTONF   =  IP.NRLANCTONF  ) 
                            AND  ( II.NRLANCTONF   =  NF.NRLANCTONF  ) 
                            AND  ( IP.NRLANCTONF   =  NF.NRLANCTONF  ) 
                            AND  ( II.NRSEQITPROD  =  IP.NRSEQITPROD ) 
                            AND  ( II.CDIMPOSTO    =  '31'           )
                            AND  ( NF.CDCLIENTE IS NOT NULL          )
                            AND  ( NF.DTENTRSAID BETWEEN @V_DTINI AND @V_DTFIM)
                            AND  ((PG.CDPRODUTO  IS NULL) OR (IP.CDPRODUTO <> PG.CDPRODUTO))  
                            AND  ((PG.CDPRAEROIN IS NULL) OR (IP.CDPRODUTO <> PG.CDPRAEROIN)) 
                            GROUP BY 
                                   NF.CDCLIENTE,
                                   IP.NRLANCTONF,
                                   CLI.NMRAZSOCCLIE,
                                   NF.DTENTRSAID           
                           ) AS HD,   
                                                                          
--------------------------------------------------------------------------------              
                     ------------VALOR TAXA AEROPORTO--------------
                           (SELECT 
	                           IP.CDFILIAL,          NF.CDCLIENTE, 
	                           CLI.NMRAZSOCCLIE,     NF.NRLANCTONF,
	                           SUM(IP.VRTOTAITEM) AS VRTXAERO,
	                           SUM(IP.VRBRUTITEM) AS VRTXAEROB
                            FROM    
                                   ITEMPRODUTO IP, PARAMCOMIS PG, NOTAFISCAL NF, CLIENTE CLI
                            WHERE  
	                         ( IP.CDFILIAL     =  PG.CDFILIAL     ) 
	                    AND  ( IP.CDFILIAL     =  @V_CDFILIAL     ) 
	                    AND  ( NF.CDCLIENTE    =  @V_CDCLIENTE    )
	                    AND  ( NF.CDCLIENTE    =  CLI.CDCLIENTE   )
	                    AND  ( NF.NRLANCTONF   =  IP.NRLANCTONF   ) 
	                    AND  ( NF.DTENTRSAID BETWEEN @V_DTINI AND @V_DTFIM ) 
	                    AND  ((IP.CDPRODUTO = PG.CDPRODUTO ) OR ( IP.CDPRODUTO = PG.CDPRAEROIN ))
	                    GROUP BY
                                   IP.CDFILIAL, 
                                   NF.CDCLIENTE, 
                                   CLI.NMRAZSOCCLIE,
                                   NF.NRLANCTONF
	                   ) AS TA,   
	                                                                                                                   
--------------------------------------------------------------------------------                         
                     ------------VALOR DE ENCARGOS-------------                                                          
                           (SELECT 
                                   IE.NRLANCTONF,        NF.CDCLIENTE, 
                                   CLI.NMRAZSOCCLIE,     NF.DTENTRSAID,
                                   SUM(IE.VRITEMENCA) AS VRITEMENCA
                            FROM   
                                   ITEMENCARGO IE, ENCARGO EN,
                                   NOTAFISCAL NF, CLIENTE CLI
                            WHERE  
	                         ( IE.CDFILIAL     =  @V_CDFILIAL    )
	                    AND  ( NF.CDCLIENTE    =  @V_CDCLIENTE   )      
	                    AND  ( NF.CDCLIENTE    =  CLI.CDCLIENTE  )
	                    AND  ( IE.NRLANCTONF   =  NF.NRLANCTONF  ) 
	                    AND  ( IE.CDENCARGO    = EN.CDENCARGO    ) 
	                    AND  ( NF.DTENTRSAID BETWEEN @V_DTINI AND @V_DTFIM) 
	                    AND  ( IE.CDENCARGO BETWEEN '06' AND '13')
	                    GROUP BY
	                           NF.CDCLIENTE,
	                           IE.NRLANCTONF,
	                           CLI.NMRAZSOCCLIE,
	                           NF.DTENTRSAID
                           ) AS IE,
                                                                                          
                           (SELECT 
                                   IE.NRLANCTONF,      NF.CDCLIENTE, 
                                   CLI.NMRAZSOCCLIE,   NF.DTENTRSAID,
                                   SUM(IE.VRITEMENCA) AS VRITEMENCAAC
                            FROM   
                                   ITEMENCARGO IE, ENCARGO EN,
                                   NOTAFISCAL NF, CLIENTE CLI
                            WHERE  
                                 ( IE.CDFILIAL     =  @V_CDFILIAL    )
                            AND  ( NF.CDCLIENTE    =  @V_CDCLIENTE   )      
                            AND  ( NF.CDCLIENTE    =  CLI.CDCLIENTE  )
                            AND  ( IE.NRLANCTONF   =  NF.NRLANCTONF  ) 
                            AND  ( IE.CDENCARGO    =  EN.CDENCARGO   ) 
                            AND  ( NF.DTENTRSAID BETWEEN @V_DTINI AND @V_DTFIM) 
                            AND  ( IE.CDENCARGO   IN ('01','03','04','05','14','15'))
                            GROUP BY
                                   NF.CDCLIENTE,
                                   IE.NRLANCTONF,
                                   CLI.NMRAZSOCCLIE,
                                   NF.DTENTRSAID
                           ) AS AC,   
                                                                                         
--------------------------------------------------------------------------------
                     -------VALOR DE IMPOSTO NOTA FISCAL---------
                           (SELECT 
                                   IT.NRLANCTONF,       NF.CDCLIENTE, 
                                   CLI.NMRAZSOCCLIE,    NF.DTENTRSAID,
                                   SUM(IT.VRTOTIMPONF) AS VRIMPOSTOS
	                    FROM    
	                           IMPOSNFTOT IT,
	                           NOTAFISCAL NF, CLIENTE CLI
	                    WHERE   
	                         ( IT.CDFILIAL   =  @V_CDFILIAL   ) 
	                    AND  ( NF.CDCLIENTE  =  @V_CDCLIENTE  )      
	                    AND  ( NF.CDCLIENTE  =  CLI.CDCLIENTE )
	                    AND  ( IT.NRLANCTONF =  NF.NRLANCTONF ) 
	                    AND  ( IT.CDIMPOSTO   IN  ('00','31','50','60'))
	                    AND  ( NF.DTENTRSAID BETWEEN @V_DTINI AND @V_DTFIM) 
	                    GROUP BY
	                           NF.CDCLIENTE, 
	                           IT.NRLANCTONF,
	                           NF.DTENTRSAID,
	                           CLI.NMRAZSOCCLIE
	                     ) AS IM       
	                                                                      
--------------------------------------------------------------------------------
                     WHERE         
                          ( NF.CDFILIAL    =  @V_CDFILIAL   )
	             AND  ( CL.CDCLIENTE   =  @V_CDCLIENTE  )
	             AND  ( NF.NRLANCTONF  =  RE.NRLANCTONF )  
	             AND  ( NF.NRLANCTONF  =  HD.NRLANCTONF )  
	             AND  ( NF.NRLANCTONF  =  TA.NRLANCTONF )
	             AND  ( NF.NRLANCTONF  =  AC.NRLANCTONF )
	             AND  ( NF.NRLANCTONF  =  IE.NRLANCTONF )
	             AND  ( NF.NRLANCTONF  =  IM.NRLANCTONF )
	             GROUP BY
	                    NF.CDFILIAL,
	                    CL.CDCLIENTE
	             );
                  OPEN C_VOICE;
                  FETCH NEXT FROM C_VOICE INTO
	                                  @V_CDFILIAL_VOICE,  @V_CDCLIENTE_VOICE,  @V_VRFATURAMENTO,
	                                  @V_VRMERCADORIALI,  @V_VRHANDLINGLI,     @V_VRTXAERO,              
	                                  @V_VRITEMENCA ,     @V_VRITEMENCAAC,     @V_VRIMPOSTOS;
                  WHILE(@@FETCH_STATUS = 0) BEGIN                            
--------------------------------------------------------------------------------
                                                                         
	                  SET @V_MES =  CONVERT(VARCHAR, MONTH(@V_DTFIM));                                    
	                  SET @V_ANO =  CONVERT(VARCHAR, YEAR (@V_DTFIM));                                   
	                                                                           
                          INSERT INTO  ETLFATUINVOICE                                          
	                    (  CDFILIAL,        CDEMPRESA ,      CDAREAGERE ,        
                  	       CDPAIS,          CDCLIENTE,       ANETLFAC,     
                 	       MEETLFAC,        VRFATUETLFAC,    VRMERCETLFAC,  
                	       VRSERVETLFAC,    VRTXARETLFAC,    VRDESCETLFAC,
               	               VRACREETLFAC,    VRIMPOETLFAC                                            
	                    )                                                                               
	                  VALUES                                                                    
	                    (  @V_CDFILIAL,        @V_CDEMPRESA,           @V_CDAREAGERE,  
	                       @V_CDPAIS,          @V_CDCLIENTE,           @V_ANO,            
	                       @V_MES,             @V_VRFATURAMENTO,       @V_VRMERCADORIALI,    
	                       @V_VRHANDLINGLI,    @V_VRTXAERO,            @V_VRITEMENCA,
	                       @V_VRITEMENCAAC,    @V_VRIMPOSTOS       
	                    );                                                          
	                                            
                                

                       
                          SET @V_CONT = @V_CONT + 1;                  
	                  IF @V_CONT >= 500 BEGIN
                             COMMIT TRANSACTION; 
	                     SET @V_CONT = 0;
                             BEGIN TRANSACTION; 
	                  END; /* Fim IF COMMIT para 500 */
                                                                                 
                     FETCH NEXT FROM C_VOICE INTO			                                                                        
		                                @V_CDFILIAL,       @V_CDCLIENTE,    @V_VRFATURAMENTO,
		     	                        @V_VRMERCADORIALI, @V_VRHANDLINGLI, @V_VRTXAERO,              
		     	                        @V_VRITEMENCA,     @V_VRITEMENCAAC, @V_VRIMPOSTOS;
                  END; /*FIM CURSOR C_VOICE*/
                  CLOSE C_VOICE;
                  DEALLOCATE C_VOICE;                                              
                       
               FETCH NEXT FROM C_CLIENTE INTO
                                            @V_CDCLIENTE;                       
            END;/*END LOOP CLIENTE*/
            CLOSE C_CLIENTE;
            DEALLOCATE C_CLIENTE;         
            
         END; /*End fim if date */   
         FETCH NEXT FROM C_ETL INTO /* Fim loop for C_ETL */
                                  @V_CDFILIAL,     @V_CDAREAGERE,      @V_CDSEGMER,
                                  @V_CDEMPRESA,    @V_CDSUPERVISOR,    @V_DTINICIALBI,
                                  @V_CDGOPERAC,    @V_CDPAIS,          @V_CDREGIAO; 
      END;                                  
      CLOSE C_ETL;
      DEALLOCATE C_ETL;
      
      COMMIT;
      SET @V_DTINI = DATEADD (MM, 1, @V_DTINI);
   END; /* Fim loop while data inicial / data final */
END;    /* Fim ETL_FATUINVOICE */
GO

-- VERSAO --
EXEC ATUVEROBJ 'ETL_FATUINVOICE','5.05.019';

GO

